import java.util.*;

public class ConnectedProperties
{
    public String name;
    public String basePath;
    public int[] matchBlocks;
    public String[] matchTiles;
    public int method;
    public String[] tiles;
    public int connect;
    public int faces;
    public int[] metadatas;
    public aav[] biomes;
    public int minHeight;
    public int maxHeight;
    public int renderPass;
    public boolean innerSeams;
    public int width;
    public int height;
    public int[] weights;
    public int symmetry;
    public int[] sumWeights;
    public int sumAllWeights;
    public lx[] matchTileIcons;
    public lx[] tileIcons;
    public static final int METHOD_NONE = 0;
    public static final int METHOD_CTM = 1;
    public static final int METHOD_HORIZONTAL = 2;
    public static final int METHOD_TOP = 3;
    public static final int METHOD_RANDOM = 4;
    public static final int METHOD_REPEAT = 5;
    public static final int METHOD_VERTICAL = 6;
    public static final int METHOD_FIXED = 7;
    public static final int CONNECT_NONE = 0;
    public static final int CONNECT_BLOCK = 1;
    public static final int CONNECT_TILE = 2;
    public static final int CONNECT_MATERIAL = 3;
    public static final int CONNECT_UNKNOWN = 128;
    public static final int FACE_BOTTOM = 1;
    public static final int FACE_TOP = 2;
    public static final int FACE_EAST = 4;
    public static final int FACE_WEST = 8;
    public static final int FACE_NORTH = 16;
    public static final int FACE_SOUTH = 32;
    public static final int FACE_SIDES = 60;
    public static final int FACE_ALL = 63;
    public static final int FACE_UNKNOWN = 128;
    public static final int SYMMETRY_NONE = 1;
    public static final int SYMMETRY_OPPOSITE = 2;
    public static final int SYMMETRY_ALL = 6;
    public static final int SYMMETRY_UNKNOWN = 128;
    
    public ConnectedProperties(final Properties props, final String path) {
        this.name = null;
        this.basePath = null;
        this.matchBlocks = null;
        this.matchTiles = null;
        this.method = 0;
        this.tiles = null;
        this.connect = 0;
        this.faces = 63;
        this.metadatas = null;
        this.biomes = null;
        this.minHeight = 0;
        this.maxHeight = 1024;
        this.renderPass = 0;
        this.innerSeams = false;
        this.width = 0;
        this.height = 0;
        this.weights = null;
        this.symmetry = 1;
        this.sumWeights = null;
        this.sumAllWeights = 0;
        this.matchTileIcons = null;
        this.tileIcons = null;
        this.name = parseName(path);
        this.basePath = parseBasePath(path);
        this.matchBlocks = parseInts(props.getProperty("matchBlocks"));
        this.matchTiles = this.parseMatchTiles(props.getProperty("matchTiles"));
        this.method = parseMethod(props.getProperty("method"));
        this.tiles = this.parseTileNames(props.getProperty("tiles"));
        this.connect = parseConnect(props.getProperty("connect"));
        this.faces = parseFaces(props.getProperty("faces"));
        this.metadatas = parseInts(props.getProperty("metadata"));
        this.biomes = parseBiomes(props.getProperty("biomes"));
        this.minHeight = parseInt(props.getProperty("minHeight"), -1);
        this.maxHeight = parseInt(props.getProperty("maxHeight"), 1024);
        this.renderPass = parseInt(props.getProperty("renderPass"));
        this.innerSeams = parseBoolean(props.getProperty("innerSeams"));
        this.width = parseInt(props.getProperty("width"));
        this.height = parseInt(props.getProperty("height"));
        this.weights = parseInts(props.getProperty("weights"));
        this.symmetry = parseSymmetry(props.getProperty("symmetry"));
    }
    
    private String[] parseMatchTiles(final String str) {
        if (str == null) {
            return null;
        }
        final String[] names = Config.tokenize(str, " ");
        for (int i = 0; i < names.length; ++i) {
            String iconName = names[i];
            if (iconName.endsWith(".png")) {
                iconName = iconName.substring(0, iconName.length() - 4);
            }
            if (iconName.startsWith("/ctm/")) {
                iconName = iconName.substring(1);
            }
            names[i] = iconName;
        }
        return names;
    }
    
    private static String parseName(final String path) {
        String str = path;
        final int pos = str.lastIndexOf(47);
        if (pos >= 0) {
            str = str.substring(pos + 1);
        }
        final int pos2 = str.lastIndexOf(46);
        if (pos2 >= 0) {
            str = str.substring(0, pos2);
        }
        return str;
    }
    
    private static String parseBasePath(final String path) {
        final int pos = path.lastIndexOf(47);
        if (pos < 0) {
            return "";
        }
        return path.substring(0, pos);
    }
    
    private static aav[] parseBiomes(final String str) {
        if (str == null) {
            return null;
        }
        final String[] biomeNames = Config.tokenize(str, " ");
        final List list = new ArrayList();
        for (int i = 0; i < biomeNames.length; ++i) {
            final String biomeName = biomeNames[i];
            final aav biome = findBiome(biomeName);
            if (biome == null) {
                Config.dbg("Biome not found: " + biomeName);
            }
            else {
                list.add(biome);
            }
        }
        final aav[] biomeArr = list.toArray(new aav[list.size()]);
        return biomeArr;
    }
    
    private static aav findBiome(String biomeName) {
        biomeName = biomeName.toLowerCase();
        for (int i = 0; i < aav.a.length; ++i) {
            final aav biome = aav.a[i];
            if (biome != null) {
                final String name = biome.y.replace(" ", "").toLowerCase();
                if (name.equals(biomeName)) {
                    return biome;
                }
            }
        }
        return null;
    }
    
    private String[] parseTileNames(final String str) {
        if (str == null) {
            return null;
        }
        final List list = new ArrayList();
        final String[] iconStrs = Config.tokenize(str, " ,");
        for (int i = 0; i < iconStrs.length; ++i) {
            final String iconStr = iconStrs[i];
            if (iconStr.contains("-")) {
                final String[] subStrs = Config.tokenize(iconStr, "-");
                if (subStrs.length == 2) {
                    final int min = Config.parseInt(subStrs[0], -1);
                    final int max = Config.parseInt(subStrs[1], -1);
                    if (min >= 0 && max >= 0) {
                        if (min > max) {
                            Config.dbg("Invalid interval: " + iconStr + ", when parsing: " + str);
                            continue;
                        }
                        for (int n = min; n <= max; ++n) {
                            list.add(String.valueOf(n));
                        }
                        continue;
                    }
                }
            }
            list.add(iconStr);
        }
        final String[] names = list.toArray(new String[list.size()]);
        for (int j = 0; j < names.length; ++j) {
            String iconName = names[j];
            if (!iconName.startsWith("/")) {
                iconName = this.basePath + "/" + iconName;
            }
            if (iconName.endsWith(".png")) {
                iconName = iconName.substring(0, iconName.length() - 4);
            }
            final String pathBlocks = "/textures/blocks/";
            if (iconName.startsWith(pathBlocks)) {
                iconName = iconName.substring(pathBlocks.length());
            }
            if (iconName.startsWith("/")) {
                iconName = iconName.substring(1);
            }
            names[j] = iconName;
        }
        return names;
    }
    
    private static int parseInt(final String str) {
        if (str == null) {
            return -1;
        }
        final int num = Config.parseInt(str, -1);
        if (num < 0) {
            Config.dbg("Invalid number: " + str);
        }
        return num;
    }
    
    private static int parseInt(final String str, final int defVal) {
        if (str == null) {
            return defVal;
        }
        final int num = Config.parseInt(str, -1);
        if (num < 0) {
            Config.dbg("Invalid number: " + str);
            return defVal;
        }
        return num;
    }
    
    private static boolean parseBoolean(final String str) {
        return str != null && str.toLowerCase().equals("true");
    }
    
    private static int parseSymmetry(final String str) {
        if (str == null) {
            return 1;
        }
        if (str.equals("opposite")) {
            return 2;
        }
        if (str.equals("all")) {
            return 6;
        }
        Config.dbg("Unknown symmetry: " + str);
        return 1;
    }
    
    private static int parseFaces(final String str) {
        if (str == null) {
            return 63;
        }
        final String[] faceStrs = Config.tokenize(str, " ,");
        int facesMask = 0;
        for (int i = 0; i < faceStrs.length; ++i) {
            final String faceStr = faceStrs[i];
            final int faceMask = parseFace(faceStr);
            facesMask |= faceMask;
        }
        return facesMask;
    }
    
    private static int parseFace(final String str) {
        if (str.equals("bottom")) {
            return 1;
        }
        if (str.equals("top")) {
            return 2;
        }
        if (str.equals("north")) {
            return 4;
        }
        if (str.equals("south")) {
            return 8;
        }
        if (str.equals("east")) {
            return 32;
        }
        if (str.equals("west")) {
            return 16;
        }
        if (str.equals("sides")) {
            return 60;
        }
        if (str.equals("all")) {
            return 63;
        }
        Config.dbg("Unknown face: " + str);
        return 128;
    }
    
    private static int parseConnect(final String str) {
        if (str == null) {
            return 0;
        }
        if (str.equals("block")) {
            return 1;
        }
        if (str.equals("tile")) {
            return 2;
        }
        if (str.equals("material")) {
            return 3;
        }
        Config.dbg("Unknown connect: " + str);
        return 128;
    }
    
    private static int[] parseInts(final String str) {
        if (str == null) {
            return null;
        }
        final List list = new ArrayList();
        final String[] intStrs = Config.tokenize(str, " ,");
        for (int i = 0; i < intStrs.length; ++i) {
            final String intStr = intStrs[i];
            if (intStr.contains("-")) {
                final String[] subStrs = Config.tokenize(intStr, "-");
                if (subStrs.length != 2) {
                    Config.dbg("Invalid interval: " + intStr + ", when parsing: " + str);
                }
                else {
                    final int min = Config.parseInt(subStrs[0], -1);
                    final int max = Config.parseInt(subStrs[1], -1);
                    if (min < 0 || max < 0 || min > max) {
                        Config.dbg("Invalid interval: " + intStr + ", when parsing: " + str);
                    }
                    else {
                        for (int n = min; n <= max; ++n) {
                            list.add(n);
                        }
                    }
                }
            }
            else {
                final int val = Config.parseInt(intStr, -1);
                if (val < 0) {
                    Config.dbg("Invalid number: " + intStr + ", when parsing: " + str);
                }
                else {
                    list.add(val);
                }
            }
        }
        final int[] ints = new int[list.size()];
        for (int j = 0; j < ints.length; ++j) {
            ints[j] = list.get(j);
        }
        return ints;
    }
    
    private static int parseMethod(final String str) {
        if (str == null) {
            return 1;
        }
        if (str.equals("ctm")) {
            return 1;
        }
        if (str.equals("horizontal")) {
            return 2;
        }
        if (str.equals("vertical")) {
            return 6;
        }
        if (str.equals("top")) {
            return 3;
        }
        if (str.equals("random")) {
            return 4;
        }
        if (str.equals("repeat")) {
            return 5;
        }
        if (str.equals("fixed")) {
            return 7;
        }
        Config.dbg("Unknown method: " + str);
        return 0;
    }
    
    public boolean isValid(final String path) {
        if (this.name == null || this.name.length() <= 0) {
            Config.dbg("No name found: " + path);
            return false;
        }
        if (this.basePath == null) {
            Config.dbg("No base path found: " + path);
            return false;
        }
        if (this.matchBlocks == null) {
            this.matchBlocks = this.detectMatchBlocks();
        }
        if (this.matchTiles == null && this.matchBlocks == null) {
            this.matchTiles = this.detectMatchTiles();
        }
        if (this.matchBlocks == null && this.matchTiles == null) {
            Config.dbg("No matchBlocks or matchTiles specified: " + path);
            return false;
        }
        if (this.method == 0) {
            Config.dbg("No method: " + path);
            return false;
        }
        if (this.tiles == null || this.tiles.length <= 0) {
            Config.dbg("No tiles specified: " + path);
            return false;
        }
        if (this.connect == 0) {
            this.connect = this.detectConnect();
        }
        if (this.connect == 128) {
            Config.dbg("Invalid connect in: " + path);
            return false;
        }
        if (this.renderPass > 0) {
            Config.dbg("Render pass not supported: " + this.renderPass);
            return false;
        }
        if ((this.faces & 0x80) != 0x0) {
            Config.dbg("Invalid faces in: " + path);
            return false;
        }
        if ((this.symmetry & 0x80) != 0x0) {
            Config.dbg("Invalid symmetry in: " + path);
            return false;
        }
        switch (this.method) {
            case 1: {
                return this.isValidCtm(path);
            }
            case 2: {
                return this.isValidHorizontal(path);
            }
            case 6: {
                return this.isValidVertical(path);
            }
            case 3: {
                return this.isValidTop(path);
            }
            case 4: {
                return this.isValidRandom(path);
            }
            case 5: {
                return this.isValidRepeat(path);
            }
            case 7: {
                return this.isValidFixed(path);
            }
            default: {
                Config.dbg("Unknown method: " + path);
                return false;
            }
        }
    }
    
    private int detectConnect() {
        if (this.matchBlocks != null) {
            return 1;
        }
        if (this.matchTiles != null) {
            return 2;
        }
        return 128;
    }
    
    private int[] detectMatchBlocks() {
        if (!this.name.startsWith("block")) {
            return null;
        }
        int pos;
        int startPos;
        for (startPos = (pos = "block".length()); pos < this.name.length(); ++pos) {
            final char ch = this.name.charAt(pos);
            if (ch < '0') {
                break;
            }
            if (ch > '9') {
                break;
            }
        }
        if (pos == startPos) {
            return null;
        }
        final String idStr = this.name.substring(startPos, pos);
        final int id = Config.parseInt(idStr, -1);
        if (id < 0) {
            return null;
        }
        return new int[] { id };
    }
    
    private String[] detectMatchTiles() {
        final lx icon = getIcon(this.name);
        if (icon == null) {
            return null;
        }
        return new String[] { this.name };
    }
    
    private static lx getIcon(final String iconName) {
        final bge re = Config.getRenderEngine();
        if (re == null) {
            return null;
        }
        return re.i.getIconSafe(iconName);
    }
    
    private boolean isValidCtm(final String path) {
        if (this.tiles == null) {
            this.tiles = this.parseTileNames("0-11 16-27 32-43 48-58");
        }
        if (this.tiles.length < 47) {
            Config.dbg("Invalid tiles, must be at least 47: " + path);
            return false;
        }
        return true;
    }
    
    private boolean isValidHorizontal(final String path) {
        if (this.tiles == null) {
            this.tiles = this.parseTileNames("12-15");
        }
        if (this.tiles.length != 4) {
            Config.dbg("Invalid tiles, must be exactly 4: " + path);
            return false;
        }
        return true;
    }
    
    private boolean isValidVertical(final String path) {
        if (this.tiles == null) {
            Config.dbg("No tiles defined for vertical: " + path);
            return false;
        }
        if (this.tiles.length != 4) {
            Config.dbg("Invalid tiles, must be exactly 4: " + path);
            return false;
        }
        return true;
    }
    
    private boolean isValidRandom(final String path) {
        if (this.tiles == null || this.tiles.length <= 0) {
            Config.dbg("Tiles not defined: " + path);
            return false;
        }
        if (this.weights != null && this.weights.length != this.tiles.length) {
            Config.dbg("Number of weights must equal the number of tiles: " + path);
            this.weights = null;
        }
        if (this.weights != null) {
            this.sumWeights = new int[this.weights.length];
            int sum = 0;
            for (int i = 0; i < this.weights.length; ++i) {
                sum += this.weights[i];
                this.sumWeights[i] = sum;
            }
            this.sumAllWeights = sum;
        }
        return true;
    }
    
    private boolean isValidRepeat(final String path) {
        if (this.tiles == null) {
            Config.dbg("Tiles not defined: " + path);
            return false;
        }
        if (this.width <= 0 || this.width > 16) {
            Config.dbg("Invalid width: " + path);
            return false;
        }
        if (this.height <= 0 || this.height > 16) {
            Config.dbg("Invalid height: " + path);
            return false;
        }
        if (this.tiles.length != this.width * this.height) {
            Config.dbg("Number of tiles does not equal width x height: " + path);
            return false;
        }
        return true;
    }
    
    private boolean isValidFixed(final String path) {
        if (this.tiles == null) {
            Config.dbg("Tiles not defined: " + path);
            return false;
        }
        if (this.tiles.length != 1) {
            Config.dbg("Number of tiles should be 1 for method: fixed.");
            return false;
        }
        return true;
    }
    
    private boolean isValidTop(final String path) {
        if (this.tiles == null) {
            this.tiles = this.parseTileNames("66");
        }
        if (this.tiles.length != 1) {
            Config.dbg("Invalid tiles, must be exactly 1: " + path);
            return false;
        }
        return true;
    }
    
    public void updateIcons(final bir textureMap) {
        if (this.matchTiles != null) {
            this.matchTileIcons = registerIcons(this.matchTiles, textureMap);
        }
        if (this.tiles != null) {
            this.tileIcons = registerIcons(this.tiles, textureMap);
        }
    }
    
    private static lx[] registerIcons(final String[] tileNames, final bir textureMap) {
        if (tileNames == null) {
            return null;
        }
        final bjt tp = Config.getRenderEngine().getTexturePack().e();
        final List iconList = new ArrayList();
        for (int i = 0; i < tileNames.length; ++i) {
            String fullName;
            final String iconName = fullName = tileNames[i];
            if (!fullName.contains("/")) {
                fullName = "textures/blocks/" + fullName;
            }
            final String fileName = "/" + fullName + ".png";
            final boolean exists = tp.b(fileName, true);
            if (!exists) {
                Config.dbg("File not found: " + fileName);
            }
            final lx icon = textureMap.a(iconName);
            iconList.add(icon);
        }
        final lx[] icons = iconList.toArray(new lx[iconList.size()]);
        return icons;
    }
    
    public String toString() {
        return "CTM name: " + this.name + ", basePath: " + this.basePath + ", matchBlocks: " + Config.arrayToString(this.matchBlocks) + ", matchTiles: " + Config.arrayToString(this.matchTiles);
    }
}
