import java.awt.image.*;
import java.io.*;
import java.util.*;

public class CustomColorizer
{
    private static int[] grassColors;
    private static int[] waterColors;
    private static int[] foliageColors;
    private static int[] foliagePineColors;
    private static int[] foliageBirchColors;
    private static int[] swampFoliageColors;
    private static int[] swampGrassColors;
    private static int[][] blockPalettes;
    private static int[][] paletteColors;
    private static int[] skyColors;
    private static int[] fogColors;
    private static int[] underwaterColors;
    private static float[][][] lightMapsColorsRgb;
    private static int[] lightMapsHeight;
    private static float[][] sunRgbs;
    private static float[][] torchRgbs;
    private static int[] redstoneColors;
    private static int[] stemColors;
    private static int[] myceliumParticleColors;
    private static boolean useDefaultColorMultiplier;
    private static int particleWaterColor;
    private static int particlePortalColor;
    private static int lilyPadColor;
    private static arc fogColorNether;
    private static arc fogColorEnd;
    private static arc skyColorEnd;
    private static final int TYPE_NONE = 0;
    private static final int TYPE_GRASS = 1;
    private static final int TYPE_FOLIAGE = 2;
    private static Random random;
    
    public static void update(final bge re) {
        CustomColorizer.grassColors = null;
        CustomColorizer.waterColors = null;
        CustomColorizer.foliageColors = null;
        CustomColorizer.foliageBirchColors = null;
        CustomColorizer.foliagePineColors = null;
        CustomColorizer.swampGrassColors = null;
        CustomColorizer.swampFoliageColors = null;
        CustomColorizer.skyColors = null;
        CustomColorizer.fogColors = null;
        CustomColorizer.underwaterColors = null;
        CustomColorizer.redstoneColors = null;
        CustomColorizer.stemColors = null;
        CustomColorizer.myceliumParticleColors = null;
        CustomColorizer.lightMapsColorsRgb = null;
        CustomColorizer.lightMapsHeight = null;
        CustomColorizer.lilyPadColor = -1;
        CustomColorizer.particleWaterColor = -1;
        CustomColorizer.particlePortalColor = -1;
        CustomColorizer.fogColorNether = null;
        CustomColorizer.fogColorEnd = null;
        CustomColorizer.skyColorEnd = null;
        CustomColorizer.blockPalettes = null;
        CustomColorizer.paletteColors = null;
        CustomColorizer.useDefaultColorMultiplier = true;
        CustomColorizer.grassColors = getCustomColors("/misc/grasscolor.png", re, 65536);
        CustomColorizer.foliageColors = getCustomColors("/misc/foliagecolor.png", re, 65536);
        CustomColorizer.waterColors = getCustomColors("/misc/watercolorX.png", re, 65536);
        if (!Config.isCustomColors()) {
            return;
        }
        CustomColorizer.foliagePineColors = getCustomColors("/misc/pinecolor.png", re, 65536);
        CustomColorizer.foliageBirchColors = getCustomColors("/misc/birchcolor.png", re, 65536);
        CustomColorizer.swampGrassColors = getCustomColors("/misc/swampgrasscolor.png", re, 65536);
        CustomColorizer.swampFoliageColors = getCustomColors("/misc/swampfoliagecolor.png", re, 65536);
        CustomColorizer.skyColors = getCustomColors("/misc/skycolor0.png", re, 65536);
        CustomColorizer.fogColors = getCustomColors("/misc/fogcolor0.png", re, 65536);
        CustomColorizer.underwaterColors = getCustomColors("/misc/underwatercolor.png", re, 65536);
        CustomColorizer.redstoneColors = getCustomColors("/misc/redstonecolor.png", re, 16);
        CustomColorizer.stemColors = getCustomColors("/misc/stemcolor.png", re, 8);
        CustomColorizer.myceliumParticleColors = getCustomColors("/misc/myceliumparticlecolor.png", re, -1);
        final int[][] lightMapsColors = new int[3][];
        CustomColorizer.lightMapsColorsRgb = new float[3][][];
        CustomColorizer.lightMapsHeight = new int[3];
        for (int i = 0; i < lightMapsColors.length; ++i) {
            final String path = "/environment/lightmap" + (i - 1) + ".png";
            lightMapsColors[i] = getCustomColors(path, re, -1);
            if (lightMapsColors[i] != null) {
                CustomColorizer.lightMapsColorsRgb[i] = toRgb(lightMapsColors[i]);
            }
            CustomColorizer.lightMapsHeight[i] = getTextureHeight(re, path, 32);
        }
        readColorProperties("/color.properties", re);
        updateUseDefaultColorMultiplier();
    }
    
    private static int getTextureHeight(final bge re, final String path, final int defHeight) {
        try {
            final BufferedImage bi = re.readTextureImage(path);
            if (bi == null) {
                return defHeight;
            }
            return bi.getHeight();
        }
        catch (IOException e) {
            return defHeight;
        }
    }
    
    private static float[][] toRgb(final int[] cols) {
        final float[][] colsRgb = new float[cols.length][3];
        for (int i = 0; i < cols.length; ++i) {
            final int col = cols[i];
            final float rf = (col >> 16 & 0xFF) / 255.0f;
            final float gf = (col >> 8 & 0xFF) / 255.0f;
            final float bf = (col & 0xFF) / 255.0f;
            final float[] colRgb = colsRgb[i];
            colRgb[0] = rf;
            colRgb[1] = gf;
            colRgb[2] = bf;
        }
        return colsRgb;
    }
    
    private static void readColorProperties(final String fileName, final bge re) {
        try {
            final InputStream in = re.getTexturePack().e().a(fileName);
            if (in == null) {
                return;
            }
            Config.log("Loading " + fileName);
            final Properties props = new Properties();
            props.load(in);
            CustomColorizer.lilyPadColor = readColor(props, "lilypad");
            CustomColorizer.particleWaterColor = readColor(props, new String[] { "particle.water", "drop.water" });
            CustomColorizer.particlePortalColor = readColor(props, "particle.portal");
            CustomColorizer.fogColorNether = readColorVec3(props, "fog.nether");
            CustomColorizer.fogColorEnd = readColorVec3(props, "fog.end");
            CustomColorizer.skyColorEnd = readColorVec3(props, "sky.end");
            readCustomPalettes(props, re);
        }
        catch (FileNotFoundException e2) {}
        catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    private static void readCustomPalettes(final Properties props, final bge re) {
        CustomColorizer.blockPalettes = new int[256][1];
        for (int i = 0; i < 256; ++i) {
            CustomColorizer.blockPalettes[i][0] = -1;
        }
        final String palettePrefix = "palette.block.";
        final Map map = new HashMap();
        final Set keys = props.keySet();
        for (final String key : keys) {
            final String value = props.getProperty(key);
            if (!key.startsWith(palettePrefix)) {
                continue;
            }
            map.put(key, value);
        }
        final String[] propNames = (String[])map.keySet().toArray(new String[map.size()]);
        CustomColorizer.paletteColors = new int[propNames.length][];
        for (int j = 0; j < propNames.length; ++j) {
            final String name = propNames[j];
            final String value2 = props.getProperty(name);
            Config.log("Block palette: " + name + " = " + value2);
            final String path = name.substring(palettePrefix.length());
            final int[] colors = getCustomColors(path, re, 65536);
            CustomColorizer.paletteColors[j] = colors;
            final String[] indexStrs = Config.tokenize(value2, " ,;");
            for (int ix = 0; ix < indexStrs.length; ++ix) {
                String blockStr = indexStrs[ix];
                int metadata = -1;
                if (blockStr.contains(":")) {
                    final String[] blockStrs = Config.tokenize(blockStr, ":");
                    blockStr = blockStrs[0];
                    final String metadataStr = blockStrs[1];
                    metadata = Config.parseInt(metadataStr, -1);
                    if (metadata < 0 || metadata > 15) {
                        Config.log("Invalid block metadata: " + blockStr + " in palette: " + name);
                        continue;
                    }
                }
                final int blockIndex = Config.parseInt(blockStr, -1);
                if (blockIndex < 0 || blockIndex > 255) {
                    Config.log("Invalid block index: " + blockIndex + " in palette: " + name);
                }
                else if (blockIndex != apa.y.cz && blockIndex != apa.ab.cz && blockIndex != apa.O.cz) {
                    if (blockIndex != apa.by.cz) {
                        if (metadata == -1) {
                            CustomColorizer.blockPalettes[blockIndex][0] = j;
                        }
                        else {
                            if (CustomColorizer.blockPalettes[blockIndex].length < 16) {
                                Arrays.fill(CustomColorizer.blockPalettes[blockIndex] = new int[16], -1);
                            }
                            CustomColorizer.blockPalettes[blockIndex][metadata] = j;
                        }
                    }
                }
            }
        }
    }
    
    private static int readColor(final Properties props, final String[] names) {
        for (int i = 0; i < names.length; ++i) {
            final String name = names[i];
            final int col = readColor(props, name);
            if (col >= 0) {
                return col;
            }
        }
        return -1;
    }
    
    private static int readColor(final Properties props, final String name) {
        final String str = props.getProperty(name);
        if (str == null) {
            return -1;
        }
        try {
            final int val = Integer.parseInt(str, 16) & 0xFFFFFF;
            Config.log("Custom color: " + name + " = " + str);
            return val;
        }
        catch (NumberFormatException e) {
            Config.log("Invalid custom color: " + name + " = " + str);
            return -1;
        }
    }
    
    private static arc readColorVec3(final Properties props, final String name) {
        final int col = readColor(props, name);
        if (col < 0) {
            return null;
        }
        final int red = col >> 16 & 0xFF;
        final int green = col >> 8 & 0xFF;
        final int blue = col & 0xFF;
        final float redF = red / 255.0f;
        final float greenF = green / 255.0f;
        final float blueF = blue / 255.0f;
        return arc.a(redF, greenF, blueF);
    }
    
    private static int[] getCustomColors(final String path, final bge re, final int length) {
        try {
            final InputStream in = re.getTexturePack().e().a(path);
            if (in == null) {
                return null;
            }
            final int[] colors = re.a(path);
            if (colors == null) {
                return null;
            }
            if (length > 0 && colors.length != length) {
                Config.log("Invalid custom colors length: " + colors.length + ", path: " + path);
                return null;
            }
            Config.log("Loading custom colors: " + path);
            return colors;
        }
        catch (FileNotFoundException e2) {
            return null;
        }
        catch (IOException e) {
            e.printStackTrace();
            return null;
        }
    }
    
    public static void updateUseDefaultColorMultiplier() {
        CustomColorizer.useDefaultColorMultiplier = (CustomColorizer.foliageBirchColors == null && CustomColorizer.foliagePineColors == null && CustomColorizer.swampGrassColors == null && CustomColorizer.swampFoliageColors == null && CustomColorizer.blockPalettes == null && Config.isSwampColors() && Config.isSmoothBiomes());
    }
    
    public static int getColorMultiplier(final apa block, final aak blockAccess, final int x, final int y, final int z) {
        if (CustomColorizer.useDefaultColorMultiplier) {
            return block.c(blockAccess, x, y, z);
        }
        int[] colors = null;
        int[] swampColors = null;
        if (CustomColorizer.blockPalettes != null) {
            final int blockId = block.cz;
            if (blockId >= 0 && blockId < 256) {
                final int[] metadataPals = CustomColorizer.blockPalettes[blockId];
                int paletteIx = -1;
                if (metadataPals.length > 1) {
                    final int metadata = blockAccess.h(x, y, z);
                    paletteIx = metadataPals[metadata];
                }
                else {
                    paletteIx = metadataPals[0];
                }
                if (paletteIx >= 0) {
                    colors = CustomColorizer.paletteColors[paletteIx];
                }
            }
            if (colors != null) {
                if (Config.isSmoothBiomes()) {
                    return getSmoothColorMultiplier(block, blockAccess, x, y, z, colors, colors, 0, 0);
                }
                return getCustomColor(colors, blockAccess, x, y, z);
            }
        }
        final boolean useSwampColors = Config.isSwampColors();
        boolean smoothColors = false;
        int type = 0;
        int metadata = 0;
        if (block == apa.y || block == apa.ab) {
            type = 1;
            smoothColors = Config.isSmoothBiomes();
            colors = CustomColorizer.grassColors;
            if (useSwampColors) {
                swampColors = CustomColorizer.swampGrassColors;
            }
            else {
                swampColors = colors;
            }
        }
        else if (block == apa.O) {
            type = 2;
            smoothColors = Config.isSmoothBiomes();
            metadata = blockAccess.h(x, y, z);
            if ((metadata & 0x3) == 0x1) {
                colors = CustomColorizer.foliagePineColors;
            }
            else if ((metadata & 0x3) == 0x2) {
                colors = CustomColorizer.foliageBirchColors;
            }
            else {
                colors = CustomColorizer.foliageColors;
                if (useSwampColors) {
                    swampColors = CustomColorizer.swampFoliageColors;
                }
                else {
                    swampColors = colors;
                }
            }
        }
        else if (block == apa.by) {
            type = 2;
            smoothColors = Config.isSmoothBiomes();
            colors = CustomColorizer.foliageColors;
            if (useSwampColors) {
                swampColors = CustomColorizer.swampFoliageColors;
            }
            else {
                swampColors = colors;
            }
        }
        if (smoothColors) {
            return getSmoothColorMultiplier(block, blockAccess, x, y, z, colors, swampColors, type, metadata);
        }
        if (swampColors != colors && blockAccess.a(x, z) == aav.h) {
            colors = swampColors;
        }
        if (colors != null) {
            return getCustomColor(colors, blockAccess, x, y, z);
        }
        return block.c(blockAccess, x, y, z);
    }
    
    private static int getSmoothColorMultiplier(final apa block, final aak blockAccess, final int x, final int y, final int z, final int[] colors, final int[] swampColors, final int type, final int metadata) {
        int sumRed = 0;
        int sumGreen = 0;
        int sumBlue = 0;
        for (int ix = x - 1; ix <= x + 1; ++ix) {
            for (int iz = z - 1; iz <= z + 1; ++iz) {
                int[] cols = colors;
                if (swampColors != cols && blockAccess.a(ix, iz) == aav.h) {
                    cols = swampColors;
                }
                int col = 0;
                if (cols == null) {
                    switch (type) {
                        case 1: {
                            col = blockAccess.a(ix, iz).k();
                            break;
                        }
                        case 2: {
                            if ((metadata & 0x3) == 0x1) {
                                col = zx.a();
                                break;
                            }
                            if ((metadata & 0x3) == 0x2) {
                                col = zx.b();
                                break;
                            }
                            col = blockAccess.a(ix, iz).l();
                            break;
                        }
                        default: {
                            col = block.c(blockAccess, ix, y, iz);
                            break;
                        }
                    }
                }
                else {
                    col = getCustomColor(cols, blockAccess, ix, y, iz);
                }
                sumRed += (col >> 16 & 0xFF);
                sumGreen += (col >> 8 & 0xFF);
                sumBlue += (col & 0xFF);
            }
        }
        final int r = sumRed / 9;
        final int g = sumGreen / 9;
        final int b = sumBlue / 9;
        return r << 16 | g << 8 | b;
    }
    
    public static int getFluidColor(final apa block, final aak blockAccess, final int x, final int y, final int z) {
        if (block.cO != aif.h) {
            return block.c(blockAccess, x, y, z);
        }
        if (CustomColorizer.waterColors != null) {
            if (Config.isSmoothBiomes()) {
                return getSmoothColor(CustomColorizer.waterColors, blockAccess, x, y, z, 3, 1);
            }
            return getCustomColor(CustomColorizer.waterColors, blockAccess, x, y, z);
        }
        else {
            if (!Config.isSwampColors()) {
                return 16777215;
            }
            return block.c(blockAccess, x, y, z);
        }
    }
    
    private static int getCustomColor(final int[] colors, final aak blockAccess, final int x, final int y, final int z) {
        final aav bgb = blockAccess.a(x, z);
        final double temperature = kx.a(bgb.j(), 0.0f, 1.0f);
        double rainfall = kx.a(bgb.i(), 0.0f, 1.0f);
        rainfall *= temperature;
        final int cx = (int)((1.0 - temperature) * 255.0);
        final int cy = (int)((1.0 - rainfall) * 255.0);
        return colors[cy << 8 | cx] & 0xFFFFFF;
    }
    
    public static void updatePortalFX(final bet fx) {
        if (CustomColorizer.particlePortalColor < 0) {
            return;
        }
        final int col = CustomColorizer.particlePortalColor;
        final int red = col >> 16 & 0xFF;
        final int green = col >> 8 & 0xFF;
        final int blue = col & 0xFF;
        final float redF = red / 255.0f;
        final float greenF = green / 255.0f;
        final float blueF = blue / 255.0f;
        fx.j = redF;
        fx.au = greenF;
        fx.av = blueF;
    }
    
    public static void updateMyceliumFX(final bet fx) {
        if (CustomColorizer.myceliumParticleColors == null) {
            return;
        }
        final int col = CustomColorizer.myceliumParticleColors[CustomColorizer.random.nextInt(CustomColorizer.myceliumParticleColors.length)];
        final int red = col >> 16 & 0xFF;
        final int green = col >> 8 & 0xFF;
        final int blue = col & 0xFF;
        final float redF = red / 255.0f;
        final float greenF = green / 255.0f;
        final float blueF = blue / 255.0f;
        fx.j = redF;
        fx.au = greenF;
        fx.av = blueF;
    }
    
    public static void updateReddustFX(final bet fx, final aak blockAccess, final double x, final double y, final double z) {
        if (CustomColorizer.redstoneColors == null) {
            return;
        }
        final int level = blockAccess.h((int)x, (int)y, (int)z);
        final int col = getRedstoneColor(level);
        if (col == -1) {
            return;
        }
        final int red = col >> 16 & 0xFF;
        final int green = col >> 8 & 0xFF;
        final int blue = col & 0xFF;
        final float redF = red / 255.0f;
        final float greenF = green / 255.0f;
        final float blueF = blue / 255.0f;
        fx.j = redF;
        fx.au = greenF;
        fx.av = blueF;
    }
    
    public static int getRedstoneColor(final int level) {
        if (CustomColorizer.redstoneColors == null) {
            return -1;
        }
        if (level < 0 || level > 15) {
            return -1;
        }
        return CustomColorizer.redstoneColors[level] & 0xFFFFFF;
    }
    
    public static void updateWaterFX(final bet fx, final aak blockAccess) {
        if (CustomColorizer.waterColors == null) {
            return;
        }
        final int x = (int)fx.u;
        final int y = (int)fx.v;
        final int z = (int)fx.w;
        final int col = getFluidColor(apa.F, blockAccess, x, y, z);
        final int red = col >> 16 & 0xFF;
        final int green = col >> 8 & 0xFF;
        final int blue = col & 0xFF;
        float redF = red / 255.0f;
        float greenF = green / 255.0f;
        float blueF = blue / 255.0f;
        if (CustomColorizer.particleWaterColor >= 0) {
            final int redDrop = CustomColorizer.particleWaterColor >> 16 & 0xFF;
            final int greenDrop = CustomColorizer.particleWaterColor >> 8 & 0xFF;
            final int blueDrop = CustomColorizer.particleWaterColor & 0xFF;
            redF *= redDrop / 255.0f;
            greenF *= greenDrop / 255.0f;
            blueF *= blueDrop / 255.0f;
        }
        fx.j = redF;
        fx.au = greenF;
        fx.av = blueF;
    }
    
    public static int getLilypadColor() {
        if (CustomColorizer.lilyPadColor < 0) {
            return apa.bD.o();
        }
        return CustomColorizer.lilyPadColor;
    }
    
    public static arc getFogColorNether(final arc col) {
        if (CustomColorizer.fogColorNether == null) {
            return col;
        }
        return CustomColorizer.fogColorNether;
    }
    
    public static arc getFogColorEnd(final arc col) {
        if (CustomColorizer.fogColorEnd == null) {
            return col;
        }
        return CustomColorizer.fogColorEnd;
    }
    
    public static arc getSkyColorEnd(final arc col) {
        if (CustomColorizer.skyColorEnd == null) {
            return col;
        }
        return CustomColorizer.skyColorEnd;
    }
    
    public static arc getSkyColor(final arc skyColor3d, final aak blockAccess, final double x, final double y, final double z) {
        if (CustomColorizer.skyColors == null) {
            return skyColor3d;
        }
        final int col = getSmoothColor(CustomColorizer.skyColors, blockAccess, x, y, z, 10, 1);
        final int red = col >> 16 & 0xFF;
        final int green = col >> 8 & 0xFF;
        final int blue = col & 0xFF;
        float redF = red / 255.0f;
        float greenF = green / 255.0f;
        float blueF = blue / 255.0f;
        final float cRed = (float)skyColor3d.c / 0.5f;
        final float cGreen = (float)skyColor3d.d / 0.66275f;
        final float cBlue = (float)skyColor3d.e;
        redF *= cRed;
        greenF *= cGreen;
        blueF *= cBlue;
        return arc.a(redF, greenF, blueF);
    }
    
    public static arc getFogColor(final arc fogColor3d, final aak blockAccess, final double x, final double y, final double z) {
        if (CustomColorizer.fogColors == null) {
            return fogColor3d;
        }
        final int col = getSmoothColor(CustomColorizer.fogColors, blockAccess, x, y, z, 10, 1);
        final int red = col >> 16 & 0xFF;
        final int green = col >> 8 & 0xFF;
        final int blue = col & 0xFF;
        float redF = red / 255.0f;
        float greenF = green / 255.0f;
        float blueF = blue / 255.0f;
        final float cRed = (float)fogColor3d.c / 0.753f;
        final float cGreen = (float)fogColor3d.d / 0.8471f;
        final float cBlue = (float)fogColor3d.e;
        redF *= cRed;
        greenF *= cGreen;
        blueF *= cBlue;
        return arc.a(redF, greenF, blueF);
    }
    
    public static arc getUnderwaterColor(final aak blockAccess, final double x, final double y, final double z) {
        if (CustomColorizer.underwaterColors == null) {
            return null;
        }
        final int col = getSmoothColor(CustomColorizer.underwaterColors, blockAccess, x, y, z, 10, 1);
        final int red = col >> 16 & 0xFF;
        final int green = col >> 8 & 0xFF;
        final int blue = col & 0xFF;
        final float redF = red / 255.0f;
        final float greenF = green / 255.0f;
        final float blueF = blue / 255.0f;
        return arc.a(redF, greenF, blueF);
    }
    
    public static int getSmoothColor(final int[] colors, final aak blockAccess, final double x, final double y, final double z, final int samples, final int step) {
        if (colors == null) {
            return -1;
        }
        final int x2 = (int)Math.floor(x);
        final int y2 = (int)Math.floor(y);
        final int z2 = (int)Math.floor(z);
        final int n = samples * step / 2;
        int sumRed = 0;
        int sumGreen = 0;
        int sumBlue = 0;
        int count = 0;
        for (int ix = x2 - n; ix <= x2 + n; ix += step) {
            for (int iz = z2 - n; iz <= z2 + n; iz += step) {
                final int col = getCustomColor(colors, blockAccess, ix, y2, iz);
                sumRed += (col >> 16 & 0xFF);
                sumGreen += (col >> 8 & 0xFF);
                sumBlue += (col & 0xFF);
                ++count;
            }
        }
        final int r = sumRed / count;
        final int g = sumGreen / count;
        final int b = sumBlue / count;
        return r << 16 | g << 8 | b;
    }
    
    public static int mixColors(final int c1, final int c2, final float w1) {
        if (w1 <= 0.0f) {
            return c2;
        }
        if (w1 >= 1.0f) {
            return c1;
        }
        final float w2 = 1.0f - w1;
        final int r1 = c1 >> 16 & 0xFF;
        final int g1 = c1 >> 8 & 0xFF;
        final int b1 = c1 & 0xFF;
        final int r2 = c2 >> 16 & 0xFF;
        final int g2 = c2 >> 8 & 0xFF;
        final int b2 = c2 & 0xFF;
        final int r3 = (int)(r1 * w1 + r2 * w2);
        final int g3 = (int)(g1 * w1 + g2 * w2);
        final int b3 = (int)(b1 * w1 + b2 * w2);
        return r3 << 16 | g3 << 8 | b3;
    }
    
    private static int averageColor(final int c1, final int c2) {
        final int r1 = c1 >> 16 & 0xFF;
        final int g1 = c1 >> 8 & 0xFF;
        final int b1 = c1 & 0xFF;
        final int r2 = c2 >> 16 & 0xFF;
        final int g2 = c2 >> 8 & 0xFF;
        final int b2 = c2 & 0xFF;
        final int r3 = (r1 + r2) / 2;
        final int g3 = (g1 + g2) / 2;
        final int b3 = (b1 + b2) / 2;
        return r3 << 16 | g3 << 8 | b3;
    }
    
    public static int getStemColorMultiplier(final aor blockStem, final aak blockAccess, final int x, final int y, final int z) {
        if (CustomColorizer.stemColors == null) {
            return blockStem.c(blockAccess, x, y, z);
        }
        int level = blockAccess.h(x, y, z);
        if (level < 0) {
            level = 0;
        }
        if (level >= CustomColorizer.stemColors.length) {
            level = CustomColorizer.stemColors.length - 1;
        }
        return CustomColorizer.stemColors[level];
    }
    
    public static boolean updateLightmap(final aab world, final bfq entityRenderer, final int[] lmColors, final boolean nightvision) {
        if (world == null) {
            return false;
        }
        if (CustomColorizer.lightMapsColorsRgb == null) {
            return false;
        }
        if (!Config.isCustomColors()) {
            return false;
        }
        final int worldType = world.t.h;
        if (worldType < -1 || worldType > 1) {
            return false;
        }
        final int lightMapIndex = worldType + 1;
        final float[][] lightMapRgb = CustomColorizer.lightMapsColorsRgb[lightMapIndex];
        if (lightMapRgb == null) {
            return false;
        }
        final int height = CustomColorizer.lightMapsHeight[lightMapIndex];
        if (nightvision && height < 64) {
            return false;
        }
        final int width = lightMapRgb.length / height;
        if (width < 16) {
            Config.dbg("Invalid lightmap width: " + width + " for: /environment/lightmap" + worldType + ".png");
            CustomColorizer.lightMapsColorsRgb[lightMapIndex] = null;
            return false;
        }
        int startIndex = 0;
        if (nightvision) {
            startIndex = width * 16 * 2;
        }
        float sun = 1.1666666f * (world.b(1.0f) - 0.2f);
        if (world.q > 0) {
            sun = 1.0f;
        }
        sun = Config.limitTo1(sun);
        final float sunX = sun * (width - 1);
        final float torchX = Config.limitTo1(entityRenderer.e + 0.5f) * (width - 1);
        final float gamma = Config.limitTo1(Config.getGameSettings().ak);
        final boolean hasGamma = gamma > 1.0E-4f;
        getLightMapColumn(lightMapRgb, sunX, startIndex, width, CustomColorizer.sunRgbs);
        getLightMapColumn(lightMapRgb, torchX, startIndex + 16 * width, width, CustomColorizer.torchRgbs);
        final float[] rgb = new float[3];
        for (int is = 0; is < 16; ++is) {
            for (int it = 0; it < 16; ++it) {
                for (int ic = 0; ic < 3; ++ic) {
                    float comp = Config.limitTo1(CustomColorizer.sunRgbs[is][ic] + CustomColorizer.torchRgbs[it][ic]);
                    if (hasGamma) {
                        float cg = 1.0f - comp;
                        cg = 1.0f - cg * cg * cg * cg;
                        comp = gamma * cg + (1.0f - gamma) * comp;
                    }
                    rgb[ic] = comp;
                }
                final int r = (int)(rgb[0] * 255.0f);
                final int g = (int)(rgb[1] * 255.0f);
                final int b = (int)(rgb[2] * 255.0f);
                lmColors[is * 16 + it] = (0xFF000000 | r << 16 | g << 8 | b);
            }
        }
        return true;
    }
    
    private static void getLightMapColumn(final float[][] origMap, final float x, final int offset, final int width, final float[][] colRgb) {
        final int xLow = (int)Math.floor(x);
        final int xHigh = (int)Math.ceil(x);
        if (xLow == xHigh) {
            for (int y = 0; y < 16; ++y) {
                final float[] rgbLow = origMap[offset + y * width + xLow];
                final float[] rgb = colRgb[y];
                for (int i = 0; i < 3; ++i) {
                    rgb[i] = rgbLow[i];
                }
            }
            return;
        }
        final float dLow = 1.0f - (x - xLow);
        final float dHigh = 1.0f - (xHigh - x);
        for (int y2 = 0; y2 < 16; ++y2) {
            final float[] rgbLow2 = origMap[offset + y2 * width + xLow];
            final float[] rgbHigh = origMap[offset + y2 * width + xHigh];
            final float[] rgb2 = colRgb[y2];
            for (int j = 0; j < 3; ++j) {
                rgb2[j] = rgbLow2[j] * dLow + rgbHigh[j] * dHigh;
            }
        }
    }
    
    static {
        CustomColorizer.grassColors = null;
        CustomColorizer.waterColors = null;
        CustomColorizer.foliageColors = null;
        CustomColorizer.foliagePineColors = null;
        CustomColorizer.foliageBirchColors = null;
        CustomColorizer.swampFoliageColors = null;
        CustomColorizer.swampGrassColors = null;
        CustomColorizer.blockPalettes = null;
        CustomColorizer.paletteColors = null;
        CustomColorizer.skyColors = null;
        CustomColorizer.fogColors = null;
        CustomColorizer.underwaterColors = null;
        CustomColorizer.lightMapsColorsRgb = null;
        CustomColorizer.lightMapsHeight = null;
        CustomColorizer.sunRgbs = new float[16][3];
        CustomColorizer.torchRgbs = new float[16][3];
        CustomColorizer.redstoneColors = null;
        CustomColorizer.stemColors = null;
        CustomColorizer.myceliumParticleColors = null;
        CustomColorizer.useDefaultColorMultiplier = true;
        CustomColorizer.particleWaterColor = -1;
        CustomColorizer.particlePortalColor = -1;
        CustomColorizer.lilyPadColor = -1;
        CustomColorizer.fogColorNether = null;
        CustomColorizer.fogColorEnd = null;
        CustomColorizer.skyColorEnd = null;
        CustomColorizer.random = new Random();
    }
}
