import java.io.*;
import java.util.*;

public class NaturalTextures
{
    private static bge renderEngine;
    private static NaturalProperties[] propertiesByIndex;
    
    public static void update(final bge re) {
        NaturalTextures.propertiesByIndex = new NaturalProperties[0];
        NaturalTextures.renderEngine = re;
        if (!Config.isNaturalTextures()) {
            return;
        }
        final String fileName = "/natural.properties";
        try {
            final InputStream in = re.g.e().a(fileName);
            if (in == null) {
                Config.dbg("NaturalTextures: configuration \"" + fileName + "\" not found");
                NaturalTextures.propertiesByIndex = makeDefaultProperties();
                return;
            }
            final ArrayList list = new ArrayList(256);
            final String configStr = Config.readInputStream(in);
            in.close();
            final String[] configLines = Config.tokenize(configStr, "\n\r");
            Config.dbg("Natural Textures: Parsing configuration \"" + fileName + "\"");
            for (int i = 0; i < configLines.length; ++i) {
                final String line = configLines[i].trim();
                if (!line.startsWith("#")) {
                    final String[] strs = Config.tokenize(line, "=");
                    if (strs.length != 2) {
                        Config.dbg("Natural Textures: Invalid \"" + fileName + "\" line: " + line);
                    }
                    else {
                        final String key = strs[0].trim();
                        final String type = strs[1].trim();
                        final String texName = key;
                        final bil ts = re.i.getIconSafe(texName);
                        if (ts == null) {
                            Config.dbg("Natural Textures: Texture not found: \"" + fileName + "\" line: " + line);
                        }
                        else {
                            final int tileNum = ts.getIndexInMap();
                            if (tileNum < 0) {
                                Config.dbg("Natural Textures: Invalid \"" + fileName + "\" line: " + line);
                            }
                            else {
                                final NaturalProperties props = new NaturalProperties(type);
                                if (props.isValid()) {
                                    while (list.size() <= tileNum) {
                                        list.add(null);
                                    }
                                    list.set(tileNum, props);
                                }
                            }
                        }
                    }
                }
            }
            NaturalTextures.propertiesByIndex = list.toArray(new NaturalProperties[list.size()]);
        }
        catch (FileNotFoundException e2) {
            Config.dbg("NaturalTextures: configuration \"" + fileName + "\" not found");
            NaturalTextures.propertiesByIndex = makeDefaultProperties();
        }
        catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public static NaturalProperties getNaturalProperties(final lx icon) {
        if (!(icon instanceof bil)) {
            return null;
        }
        final bil ts = (bil)icon;
        final int tileNum = ts.getIndexInMap();
        if (tileNum < 0 || tileNum >= NaturalTextures.propertiesByIndex.length) {
            return null;
        }
        final NaturalProperties props = NaturalTextures.propertiesByIndex[tileNum];
        return props;
    }
    
    private static NaturalProperties[] makeDefaultProperties() {
        if (!(NaturalTextures.renderEngine.g.e() instanceof bjq)) {
            Config.dbg("NaturalTextures: Texture pack is not default, ignoring default configuration.");
            return new NaturalProperties[0];
        }
        Config.dbg("Natural Textures: Using default configuration.");
        final List propsList = new ArrayList();
        setIconProperties(propsList, "grass_top", "4F");
        setIconProperties(propsList, "stone", "2F");
        setIconProperties(propsList, "dirt", "4F");
        setIconProperties(propsList, "grass_side", "F");
        setIconProperties(propsList, "grass_side_overlay", "F");
        setIconProperties(propsList, "stoneslab_top", "F");
        setIconProperties(propsList, "bedrock", "2F");
        setIconProperties(propsList, "sand", "4F");
        setIconProperties(propsList, "gravel", "2");
        setIconProperties(propsList, "tree_side", "2F");
        setIconProperties(propsList, "tree_top", "4F");
        setIconProperties(propsList, "oreGold", "2F");
        setIconProperties(propsList, "oreIron", "2F");
        setIconProperties(propsList, "oreCoal", "2F");
        setIconProperties(propsList, "oreDiamond", "2F");
        setIconProperties(propsList, "oreRedstone", "2F");
        setIconProperties(propsList, "oreLapis", "2F");
        setIconProperties(propsList, "obsidian", "4F");
        setIconProperties(propsList, "leaves", "2F");
        setIconProperties(propsList, "leaves_opaque", "2F");
        setIconProperties(propsList, "leaves_jungle", "2");
        setIconProperties(propsList, "leaves_jungle_opaque", "2");
        setIconProperties(propsList, "snow", "4F");
        setIconProperties(propsList, "snow_side", "F");
        setIconProperties(propsList, "cactus_side", "2F");
        setIconProperties(propsList, "clay", "4F");
        setIconProperties(propsList, "mycel_side", "F");
        setIconProperties(propsList, "mycel_top", "4F");
        setIconProperties(propsList, "farmland_wet", "2F");
        setIconProperties(propsList, "farmland_dry", "2F");
        setIconProperties(propsList, "hellrock", "4F");
        setIconProperties(propsList, "hellsand", "4F");
        setIconProperties(propsList, "lightgem", "4");
        setIconProperties(propsList, "tree_spruce", "2F");
        setIconProperties(propsList, "tree_birch", "F");
        setIconProperties(propsList, "leaves_spruce", "2F");
        setIconProperties(propsList, "leaves_spruce_opaque", "2F");
        setIconProperties(propsList, "tree_jungle", "2F");
        setIconProperties(propsList, "whiteStone", "4");
        setIconProperties(propsList, "sandstone_top", "4");
        setIconProperties(propsList, "sandstone_bottom", "4F");
        setIconProperties(propsList, "redstoneLight_lit", "4F");
        final NaturalProperties[] terrainProps = propsList.toArray(new NaturalProperties[propsList.size()]);
        return terrainProps;
    }
    
    private static void setIconProperties(final List propsList, final String iconName, final String propStr) {
        final bir terrainMap = NaturalTextures.renderEngine.i;
        final lx icon = terrainMap.a(iconName);
        if (icon == null) {
            Config.dbg("*** NaturalProperties: Icon not found: " + iconName + " ***");
            return;
        }
        if (!(icon instanceof bil)) {
            Config.dbg("*** NaturalProperties: Icon is not IconStitched: " + iconName + ": " + icon.getClass().getName() + " ***");
            return;
        }
        final bil ts = (bil)icon;
        final int index = ts.getIndexInMap();
        if (index < 0) {
            Config.dbg("*** NaturalProperties: Invalid index for icon: " + iconName + ": " + index + " ***");
            return;
        }
        while (index >= propsList.size()) {
            propsList.add(null);
        }
        final NaturalProperties props = new NaturalProperties(propStr);
        propsList.set(index, props);
    }
    
    public static void updateIcons(final bir textureMap) {
    }
    
    static {
        NaturalTextures.renderEngine = null;
        NaturalTextures.propertiesByIndex = new NaturalProperties[0];
    }
}
