import java.io.*;
import java.util.*;

public class RandomMobs
{
    private static Map textureVariantsMap;
    private static bfy renderGlobal;
    private static boolean initialized;
    private static Random random;
    private static boolean working;
    
    public static void entityLoaded(final mp entity) {
        if (!(entity instanceof ng)) {
            return;
        }
        if (entity instanceof sq) {
            return;
        }
        final ng el = (ng)entity;
        final iz ws = Config.getWorldServer();
        if (ws == null) {
            return;
        }
        final mp es = ws.a(entity.k);
        if (!(es instanceof ng)) {
            return;
        }
        final ng els = (ng)es;
        final int randomId = els.persistentId;
        el.persistentId = randomId;
    }
    
    public static void worldChanged(final aab oldWorld, final aab newWorld) {
        if (newWorld != null) {
            final List entityList = newWorld.C();
            for (int e = 0; e < entityList.size(); ++e) {
                final mp entity = entityList.get(e);
                entityLoaded(entity);
            }
        }
    }
    
    public static String getTexture(final String texture) {
        if (RandomMobs.working) {
            return texture;
        }
        try {
            RandomMobs.working = true;
            if (!RandomMobs.initialized) {
                initialize();
            }
            if (RandomMobs.renderGlobal == null) {
                return texture;
            }
            final mp entity = RandomMobs.renderGlobal.renderedEntity;
            if (entity == null) {
                return texture;
            }
            if (!(entity instanceof ng)) {
                return texture;
            }
            final ng entityLiving = (ng)entity;
            if (!texture.startsWith("/mob/")) {
                return texture;
            }
            return getTexture(texture, entityLiving.persistentId);
        }
        finally {
            RandomMobs.working = false;
        }
    }
    
    private static String getTexture(final String texture, final int randomId) {
        if (randomId <= 0) {
            return texture;
        }
        String[] texs = RandomMobs.textureVariantsMap.get(texture);
        if (texs == null) {
            texs = getTextureVariants(texture);
            RandomMobs.textureVariantsMap.put(texture, texs);
        }
        if (texs == null || texs.length <= 0) {
            return texture;
        }
        final int index = randomId % texs.length;
        final String tex = texs[index];
        return tex;
    }
    
    private static String[] getTextureVariants(final String texture) {
        final bge renderEngine = Config.getRenderEngine();
        renderEngine.f(texture);
        String[] texs = new String[0];
        final int pointPos = texture.lastIndexOf(46);
        if (pointPos < 0) {
            return texs;
        }
        final String prefix = texture.substring(0, pointPos);
        final String suffix = texture.substring(pointPos);
        final int countVariants = getCountTextureVariants(texture, prefix, suffix);
        if (countVariants <= 1) {
            return texs;
        }
        texs = new String[countVariants];
        texs[0] = texture;
        for (int i = 1; i < texs.length; ++i) {
            final int texNum = i + 1;
            final String texName = prefix + texNum + suffix;
            renderEngine.f(texs[i] = texName);
        }
        Config.dbg("RandomMobs: " + texture + ", variants: " + texs.length);
        return texs;
    }
    
    private static int getCountTextureVariants(final String texture, final String prefix, final String suffix) {
        final bge renderEngine = Config.getRenderEngine();
        final int maxNum = 1000;
        for (int num = 2; num < maxNum; ++num) {
            final String variant = prefix + num + suffix;
            try {
                final InputStream in = renderEngine.g.e().a(variant);
                if (in != null) {
                    in.close();
                    continue;
                }
            }
            catch (IOException ex) {}
            return num - 1;
        }
        return maxNum;
    }
    
    public static void resetTextures() {
        RandomMobs.textureVariantsMap.clear();
        if (Config.isRandomMobs()) {
            initialize();
        }
    }
    
    private static void initialize() {
        RandomMobs.renderGlobal = Config.getRenderGlobal();
        if (RandomMobs.renderGlobal == null) {
            return;
        }
        RandomMobs.initialized = true;
        final List list = new ArrayList();
        list.add("bat");
        list.add("cat_black");
        list.add("cat_red");
        list.add("cat_siamese");
        list.add("cavespider");
        list.add("chicken");
        list.add("cow");
        list.add("creeper");
        list.add("enderman");
        list.add("enderman_eyes");
        list.add("fire");
        list.add("ghast");
        list.add("ghast_fire");
        list.add("lava");
        list.add("ozelot");
        list.add("pig");
        list.add("pigman");
        list.add("pigzombie");
        list.add("redcow");
        list.add("saddle");
        list.add("sheep");
        list.add("sheep_fur");
        list.add("silverfish");
        list.add("skeleton");
        list.add("skeleton_wither");
        list.add("slime");
        list.add("snowman");
        list.add("spider");
        list.add("spider_eyes");
        list.add("squid");
        list.add("villager");
        list.add("villager_golem");
        list.add("wither");
        list.add("wither_invul");
        list.add("wolf");
        list.add("wolf_angry");
        list.add("wolf_collar");
        list.add("wolf_tame");
        list.add("zombie");
        list.add("zombie_villager");
        for (int i = 0; i < list.size(); ++i) {
            final String name = list.get(i);
            final String tex = "/mob/" + name + ".png";
            getTexture(tex, 100);
        }
    }
    
    static {
        RandomMobs.textureVariantsMap = new HashMap();
        RandomMobs.renderGlobal = null;
        RandomMobs.initialized = false;
        RandomMobs.random = new Random();
        RandomMobs.working = false;
    }
}
