import java.lang.reflect.*;

public class ReflectorMethod
{
    private ReflectorClass reflectorClass;
    private String targetMethodName;
    private Class[] targetMethodParameterTypes;
    private boolean checked;
    private Method targetMethod;
    
    public ReflectorMethod(final ReflectorClass reflectorClass, final String targetMethodName) {
        this(reflectorClass, targetMethodName, null);
    }
    
    public ReflectorMethod(final ReflectorClass reflectorClass, final String targetMethodName, final Class[] targetMethodParameterTypes) {
        this.reflectorClass = null;
        this.targetMethodName = null;
        this.targetMethodParameterTypes = null;
        this.checked = false;
        this.targetMethod = null;
        this.reflectorClass = reflectorClass;
        this.targetMethodName = targetMethodName;
        this.targetMethodParameterTypes = targetMethodParameterTypes;
        final Method m = this.getTargetMethod();
    }
    
    public Method getTargetMethod() {
        if (this.checked) {
            return this.targetMethod;
        }
        this.checked = true;
        final Class cls = this.reflectorClass.getTargetClass();
        if (cls == null) {
            return null;
        }
        final Method[] ms = cls.getMethods();
        for (int i = 0; i < ms.length; ++i) {
            final Method m = ms[i];
            if (m.getName().equals(this.targetMethodName)) {
                if (this.targetMethodParameterTypes != null) {
                    final Class[] types = m.getParameterTypes();
                    if (!Reflector.matchesTypes(this.targetMethodParameterTypes, types)) {
                        continue;
                    }
                }
                return this.targetMethod = m;
            }
        }
        Config.log("(Reflector) Method not pesent: " + cls.getName() + "." + this.targetMethodName);
        return null;
    }
    
    public boolean exists() {
        return this.getTargetMethod() != null;
    }
    
    public Class getReturnType() {
        final Method tm = this.getTargetMethod();
        if (tm == null) {
            return null;
        }
        return tm.getReturnType();
    }
    
    public void deactivate() {
        this.checked = true;
        this.targetMethod = null;
    }
}
