import java.util.*;

public class WrUpdaterSmooth implements IWrUpdater
{
    private long lastUpdateStartTimeNs;
    private long updateStartTimeNs;
    private long updateTimeNs;
    private WorldRendererSmooth currentUpdateRenderer;
    private int renderersUpdated;
    private int renderersFound;
    
    public WrUpdaterSmooth() {
        this.lastUpdateStartTimeNs = 0L;
        this.updateStartTimeNs = 0L;
        this.updateTimeNs = 10000000L;
        this.currentUpdateRenderer = null;
        this.renderersUpdated = 0;
        this.renderersFound = 0;
    }
    
    public void initialize() {
    }
    
    public void terminate() {
    }
    
    public bfm makeWorldRenderer(final aab worldObj, final List tileEntities, final int x, final int y, final int z, final int glRenderListBase) {
        return new WorldRendererSmooth(worldObj, tileEntities, x, y, z, glRenderListBase);
    }
    
    public boolean updateRenderers(final bfy rg, final ng entityliving, final boolean flag) {
        this.lastUpdateStartTimeNs = this.updateStartTimeNs;
        this.updateStartTimeNs = System.nanoTime();
        final long finishTimeNs = this.updateStartTimeNs + this.updateTimeNs;
        int maxNum = Config.getUpdatesPerFrame();
        if (Config.isDynamicUpdates() && !rg.isMoving(entityliving)) {
            maxNum *= 3;
        }
        this.renderersUpdated = 0;
        do {
            this.renderersFound = 0;
            this.updateRenderersImpl(rg, entityliving, flag);
        } while (this.renderersFound > 0 && System.nanoTime() - finishTimeNs < 0L);
        if (this.renderersFound > 0) {
            maxNum = Math.min(maxNum, this.renderersFound);
            final long diff = 400000L;
            if (this.renderersUpdated > maxNum) {
                this.updateTimeNs -= 2L * diff;
            }
            if (this.renderersUpdated < maxNum) {
                this.updateTimeNs += diff;
            }
        }
        else {
            this.updateTimeNs = 0L;
            this.updateTimeNs -= 200000L;
        }
        if (this.updateTimeNs < 0L) {
            this.updateTimeNs = 0L;
        }
        return this.renderersUpdated > 0;
    }
    
    private void updateRenderersImpl(final bfy rg, final ng entityliving, final boolean flag) {
        this.renderersFound = 0;
        boolean currentUpdateFinished = true;
        if (this.currentUpdateRenderer != null) {
            ++this.renderersFound;
            currentUpdateFinished = this.updateRenderer(this.currentUpdateRenderer);
            if (currentUpdateFinished) {
                ++this.renderersUpdated;
            }
        }
        if (rg.j.size() <= 0) {
            return;
        }
        final int NOT_IN_FRUSTRUM_MUL = 4;
        WorldRendererSmooth wrBest = null;
        float distSqBest = Float.MAX_VALUE;
        int indexBest = -1;
        for (int i = 0; i < rg.j.size(); ++i) {
            final WorldRendererSmooth wr = (WorldRendererSmooth)rg.j.get(i);
            if (wr != null) {
                ++this.renderersFound;
                if (!wr.q) {
                    rg.j.set(i, null);
                }
                else {
                    float distSq = wr.a(entityliving);
                    if (distSq <= 256.0f && rg.isActingNow()) {
                        wr.a();
                        wr.q = false;
                        rg.j.set(i, null);
                        ++this.renderersUpdated;
                    }
                    else {
                        if (!wr.l) {
                            distSq *= NOT_IN_FRUSTRUM_MUL;
                        }
                        if (wrBest == null) {
                            wrBest = wr;
                            distSqBest = distSq;
                            indexBest = i;
                        }
                        else if (distSq < distSqBest) {
                            wrBest = wr;
                            distSqBest = distSq;
                            indexBest = i;
                        }
                    }
                }
            }
        }
        if (this.currentUpdateRenderer != null && !currentUpdateFinished) {
            return;
        }
        if (wrBest != null) {
            rg.j.set(indexBest, null);
            if (!this.updateRenderer(wrBest)) {
                return;
            }
            ++this.renderersUpdated;
            if (System.nanoTime() > this.updateStartTimeNs + this.updateTimeNs) {
                return;
            }
            final float maxDiffDistSq = distSqBest / 5.0f;
            for (int j = 0; j < rg.j.size(); ++j) {
                final WorldRendererSmooth wr2 = (WorldRendererSmooth)rg.j.get(j);
                if (wr2 != null) {
                    float distSq2 = wr2.a(entityliving);
                    if (!wr2.l) {
                        distSq2 *= NOT_IN_FRUSTRUM_MUL;
                    }
                    final float diffDistSq = Math.abs(distSq2 - distSqBest);
                    if (diffDistSq < maxDiffDistSq) {
                        rg.j.set(j, null);
                        if (!this.updateRenderer(wr2)) {
                            return;
                        }
                        ++this.renderersUpdated;
                        if (System.nanoTime() > this.updateStartTimeNs + this.updateTimeNs) {
                            break;
                        }
                    }
                }
            }
        }
        if (this.renderersFound == 0) {
            rg.j.clear();
        }
        if (rg.j.size() > 100 && this.renderersFound < rg.j.size() * 4 / 5) {
            int dstIndex = 0;
            for (int srcIndex = 0; srcIndex < rg.j.size(); ++srcIndex) {
                final Object wr3 = rg.j.get(srcIndex);
                if (wr3 != null) {
                    if (srcIndex != dstIndex) {
                        rg.j.set(dstIndex, wr3);
                    }
                    ++dstIndex;
                }
            }
            for (int j = rg.j.size() - 1; j >= dstIndex; --j) {
                rg.j.remove(j);
            }
        }
    }
    
    private boolean updateRenderer(final WorldRendererSmooth wr) {
        final long finishTime = this.updateStartTimeNs + this.updateTimeNs;
        wr.q = false;
        final boolean ready = wr.updateRenderer(finishTime);
        if (!ready) {
            this.currentUpdateRenderer = wr;
            return false;
        }
        wr.finishUpdate();
        this.currentUpdateRenderer = null;
        return true;
    }
    
    public void finishCurrentUpdate() {
    }
    
    public void resumeBackgroundUpdates() {
    }
    
    public void pauseBackgroundUpdates() {
    }
    
    public void preRender(final bfy rg, final ng player) {
    }
    
    public void postRender() {
    }
}
