package org.lwjgl;

import java.nio.*;

public class BufferChecks
{
    public static void checkFunctionAddress(final long pointer) {
        if (pointer == 0L) {
            throw new IllegalStateException("Function is not supported");
        }
    }
    
    public static void checkNullTerminated(final ByteBuffer buf) {
        if (buf.get(buf.limit() - 1) != 0) {
            throw new IllegalArgumentException("Missing null termination");
        }
    }
    
    public static void checkNullTerminated(final ByteBuffer buf, final int count) {
        int nullFound = 0;
        for (int i = buf.position(); i < buf.limit(); ++i) {
            if (buf.get(i) == 0) {
                ++nullFound;
            }
        }
        if (nullFound < count) {
            throw new IllegalArgumentException("Missing null termination");
        }
    }
    
    public static void checkNullTerminated(final IntBuffer buf) {
        if (buf.get(buf.limit() - 1) != 0) {
            throw new IllegalArgumentException("Missing null termination");
        }
    }
    
    public static void checkNotNull(final Object o) {
        if (o == null) {
            throw new IllegalArgumentException("Null argument");
        }
    }
    
    public static void checkDirect(final ByteBuffer buf) {
        if (!buf.isDirect()) {
            throw new IllegalArgumentException("ByteBuffer is not direct");
        }
    }
    
    public static void checkDirect(final ShortBuffer buf) {
        if (!buf.isDirect()) {
            throw new IllegalArgumentException("ShortBuffer is not direct");
        }
    }
    
    public static void checkDirect(final IntBuffer buf) {
        if (!buf.isDirect()) {
            throw new IllegalArgumentException("IntBuffer is not direct");
        }
    }
    
    public static void checkDirect(final LongBuffer buf) {
        if (!buf.isDirect()) {
            throw new IllegalArgumentException("LongBuffer is not direct");
        }
    }
    
    public static void checkDirect(final FloatBuffer buf) {
        if (!buf.isDirect()) {
            throw new IllegalArgumentException("FloatBuffer is not direct");
        }
    }
    
    public static void checkDirect(final DoubleBuffer buf) {
        if (!buf.isDirect()) {
            throw new IllegalArgumentException("DoubleBuffer is not direct");
        }
    }
    
    private static void throwBufferSizeException(final Buffer buf, final int size) {
        throw new IllegalArgumentException("Number of remaining buffer elements is " + buf.remaining() + ", must be at least " + size);
    }
    
    public static void checkBufferSize(final Buffer buf, final int size) {
        if (buf.remaining() < size) {
            throwBufferSizeException(buf, size);
        }
    }
    
    public static void checkBuffer(final ByteBuffer buf, final int size) {
        checkBufferSize(buf, size);
        checkDirect(buf);
    }
    
    public static void checkBuffer(final ShortBuffer buf, final int size) {
        checkBufferSize(buf, size);
        checkDirect(buf);
    }
    
    public static void checkBuffer(final IntBuffer buf, final int size) {
        checkBufferSize(buf, size);
        checkDirect(buf);
    }
    
    public static void checkBuffer(final LongBuffer buf, final int size) {
        checkBufferSize(buf, size);
        checkDirect(buf);
    }
    
    public static void checkBuffer(final FloatBuffer buf, final int size) {
        checkBufferSize(buf, size);
        checkDirect(buf);
    }
    
    public static void checkBuffer(final DoubleBuffer buf, final int size) {
        checkBufferSize(buf, size);
        checkDirect(buf);
    }
}
