package org.lwjgl;

import java.nio.*;

public final class BufferUtils
{
    public static ByteBuffer createByteBuffer(final int size) {
        return ByteBuffer.allocateDirect(size).order(ByteOrder.nativeOrder());
    }
    
    public static ShortBuffer createShortBuffer(final int size) {
        return createByteBuffer(size << 1).asShortBuffer();
    }
    
    public static CharBuffer createCharBuffer(final int size) {
        return createByteBuffer(size << 1).asCharBuffer();
    }
    
    public static IntBuffer createIntBuffer(final int size) {
        return createByteBuffer(size << 2).asIntBuffer();
    }
    
    public static LongBuffer createLongBuffer(final int size) {
        return createByteBuffer(size << 3).asLongBuffer();
    }
    
    public static FloatBuffer createFloatBuffer(final int size) {
        return createByteBuffer(size << 2).asFloatBuffer();
    }
    
    public static DoubleBuffer createDoubleBuffer(final int size) {
        return createByteBuffer(size << 3).asDoubleBuffer();
    }
    
    public static int getElementSizeExponent(final Buffer buf) {
        if (buf instanceof ByteBuffer) {
            return 0;
        }
        if (buf instanceof ShortBuffer || buf instanceof CharBuffer) {
            return 1;
        }
        if (buf instanceof FloatBuffer || buf instanceof IntBuffer) {
            return 2;
        }
        if (buf instanceof LongBuffer || buf instanceof DoubleBuffer) {
            return 3;
        }
        throw new IllegalStateException("Unsupported buffer type: " + buf);
    }
    
    public static int getOffset(final Buffer buffer) {
        return buffer.position() << getElementSizeExponent(buffer);
    }
}
