package org.lwjgl;

import java.io.*;
import org.lwjgl.input.*;
import java.lang.reflect.*;
import java.security.*;
import java.net.*;

public final class Sys
{
    private static final String JNI_LIBRARY_NAME = "lwjgl";
    private static final String VERSION = "2.4.2";
    private static final SysImplementation implementation;
    private static final String POSTFIX64BIT = "64";
    static /* synthetic */ Class class$java$lang$String;
    static /* synthetic */ Class class$java$net$URL;
    
    private static void doLoadLibrary(final String lib_name) {
        AccessController.doPrivileged((PrivilegedAction<Object>)new PrivilegedAction() {
            public Object run() {
                final String library_path = System.getProperty("org.lwjgl.librarypath");
                if (library_path != null) {
                    System.load(library_path + File.separator + System.mapLibraryName(lib_name));
                }
                else {
                    System.loadLibrary(lib_name);
                }
                return null;
            }
        });
    }
    
    private static void loadLibrary(final String lib_name) {
        try {
            doLoadLibrary(lib_name);
        }
        catch (UnsatisfiedLinkError e3) {
            if (Sys.implementation.has64Bit()) {
                try {
                    doLoadLibrary(lib_name + "64");
                    return;
                }
                catch (UnsatisfiedLinkError e2) {
                    LWJGLUtil.log("Failed to load 64 bit library: " + e2.getMessage());
                }
            }
            throw e3;
        }
    }
    
    private static SysImplementation createImplementation() {
        switch (LWJGLUtil.getPlatform()) {
            case 1: {
                return new LinuxSysImplementation();
            }
            case 3: {
                return new WindowsSysImplementation();
            }
            case 2: {
                return new MacOSXSysImplementation();
            }
            default: {
                throw new IllegalStateException("Unsupported platform");
            }
        }
    }
    
    public static String getVersion() {
        return "2.4.2";
    }
    
    public static void initialize() {
    }
    
    public static long getTimerResolution() {
        return Sys.implementation.getTimerResolution();
    }
    
    public static long getTime() {
        return Sys.implementation.getTime() & Long.MAX_VALUE;
    }
    
    public static void alert(String title, String message) {
        final boolean grabbed = Mouse.isGrabbed();
        if (grabbed) {
            Mouse.setGrabbed(false);
        }
        if (title == null) {
            title = "";
        }
        if (message == null) {
            message = "";
        }
        Sys.implementation.alert(title, message);
        if (grabbed) {
            Mouse.setGrabbed(true);
        }
    }
    
    public static boolean openURL(final String url) {
        try {
            final Class serviceManagerClass = Class.forName("javax.jnlp.ServiceManager");
            final Method lookupMethod = AccessController.doPrivileged((PrivilegedExceptionAction<Method>)new PrivilegedExceptionAction() {
                public Object run() throws Exception {
                    return serviceManagerClass.getMethod("lookup", (Sys.class$java$lang$String == null) ? (Sys.class$java$lang$String = Sys.class$("java.lang.String")) : Sys.class$java$lang$String);
                }
            });
            final Object basicService = lookupMethod.invoke(serviceManagerClass, "javax.jnlp.BasicService");
            final Class basicServiceClass = Class.forName("javax.jnlp.BasicService");
            final Method showDocumentMethod = AccessController.doPrivileged((PrivilegedExceptionAction<Method>)new PrivilegedExceptionAction() {
                public Object run() throws Exception {
                    return basicServiceClass.getMethod("showDocument", (Sys.class$java$net$URL == null) ? (Sys.class$java$net$URL = Sys.class$("java.net.URL")) : Sys.class$java$net$URL);
                }
            });
            try {
                final Boolean ret = (Boolean)showDocumentMethod.invoke(basicService, new URL(url));
                return ret;
            }
            catch (MalformedURLException e) {
                e.printStackTrace(System.err);
                return false;
            }
        }
        catch (Exception ue) {
            return Sys.implementation.openURL(url);
        }
    }
    
    public static String getClipboard() {
        return Sys.implementation.getClipboard();
    }
    
    static /* synthetic */ Class class$(final String x0) {
        try {
            return Class.forName(x0);
        }
        catch (ClassNotFoundException x) {
            throw new NoClassDefFoundError().initCause(x);
        }
    }
    
    static {
        implementation = createImplementation();
        loadLibrary("lwjgl");
        final int native_jni_version = Sys.implementation.getJNIVersion();
        final int required_version = Sys.implementation.getRequiredJNIVersion();
        if (native_jni_version != required_version) {
            throw new LinkageError("Version mismatch: jar version is '" + required_version + "', native libary version is '" + native_jni_version + "'");
        }
        Sys.implementation.setDebug(LWJGLUtil.DEBUG);
    }
}
