package org.lwjgl;

import org.lwjgl.opengl.*;
import java.lang.reflect.*;
import java.security.*;

final class WindowsSysImplementation extends DefaultSysImplementation
{
    private static final int JNI_VERSION = 22;
    static /* synthetic */ Class class$org$lwjgl$opengl$Display;
    
    public int getRequiredJNIVersion() {
        return 22;
    }
    
    public long getTimerResolution() {
        return 1000L;
    }
    
    public long getTime() {
        return nGetTime();
    }
    
    private static native long nGetTime();
    
    public final boolean has64Bit() {
        return true;
    }
    
    private static long getHwnd() {
        if (!Display.isCreated()) {
            return 0L;
        }
        try {
            final Long hwnd_obj = AccessController.doPrivileged((PrivilegedExceptionAction<Long>)new PrivilegedExceptionAction() {
                public Object run() throws Exception {
                    final Method getImplementation_method = ((WindowsSysImplementation.class$org$lwjgl$opengl$Display == null) ? (WindowsSysImplementation.class$org$lwjgl$opengl$Display = WindowsSysImplementation.class$("org.lwjgl.opengl.Display")) : WindowsSysImplementation.class$org$lwjgl$opengl$Display).getDeclaredMethod("getImplementation", (Class[])null);
                    getImplementation_method.setAccessible(true);
                    final Object display_impl = getImplementation_method.invoke(null, (Object[])null);
                    final Class WindowsDisplay_class = Class.forName("org.lwjgl.opengl.WindowsDisplay");
                    final Method getHwnd_method = WindowsDisplay_class.getDeclaredMethod("getHwnd", (Class[])null);
                    getHwnd_method.setAccessible(true);
                    final Long hwnd = (Long)getHwnd_method.invoke(display_impl, (Object[])null);
                    return hwnd;
                }
            });
            return hwnd_obj;
        }
        catch (PrivilegedActionException e) {
            throw new Error(e);
        }
    }
    
    public void alert(final String title, final String message) {
        if (!Display.isCreated()) {
            initCommonControls();
        }
        nAlert(getHwnd(), title, message);
    }
    
    private static native void nAlert(final long p0, final String p1, final String p2);
    
    private static native void initCommonControls();
    
    public boolean openURL(final String url) {
        try {
            LWJGLUtil.execPrivileged(new String[] { "rundll32", "url.dll,FileProtocolHandler", url });
            return true;
        }
        catch (Exception e) {
            LWJGLUtil.log("Failed to open url (" + url + "): " + e.getMessage());
            return false;
        }
    }
    
    public String getClipboard() {
        return nGetClipboard();
    }
    
    private static native String nGetClipboard();
    
    static /* synthetic */ Class class$(final String x0) {
        try {
            return Class.forName(x0);
        }
        catch (ClassNotFoundException x) {
            throw new NoClassDefFoundError().initCause(x);
        }
    }
    
    static {
        Sys.initialize();
    }
}
