package org.lwjgl.input;

import java.util.*;
import net.java.games.input.*;
import org.lwjgl.*;

public class Controllers
{
    private static ArrayList controllers;
    private static int controllerCount;
    private static ArrayList events;
    private static ControllerEvent event;
    private static boolean created;
    
    public static void create() throws LWJGLException {
        if (Controllers.created) {
            return;
        }
        try {
            final ControllerEnvironment env = ControllerEnvironment.getDefaultEnvironment();
            final Controller[] found = env.getControllers();
            final ArrayList lollers = new ArrayList();
            for (int i = 0; i < found.length; ++i) {
                final Controller c = found[i];
                if (!c.getType().equals(Controller.Type.KEYBOARD) && !c.getType().equals(Controller.Type.MOUSE)) {
                    lollers.add(c);
                }
            }
            for (int length = lollers.size(), j = 0; j < length; ++j) {
                final Controller c2 = lollers.get(j);
                createController(c2);
            }
            Controllers.created = true;
        }
        catch (Throwable e) {
            throw new LWJGLException("Failed to initialise controllers", e);
        }
    }
    
    private static void createController(final Controller c) {
        final Controller[] sub = c.getControllers();
        if (sub.length == 0) {
            final JInputController controller = new JInputController(Controllers.controllerCount, c);
            Controllers.controllers.add(controller);
            ++Controllers.controllerCount;
        }
        else {
            for (int i = 0; i < sub.length; ++i) {
                createController(sub[i]);
            }
        }
    }
    
    public static org.lwjgl.input.Controller getController(final int index) {
        return Controllers.controllers.get(index);
    }
    
    public static int getControllerCount() {
        return Controllers.controllers.size();
    }
    
    public static void poll() {
        for (int i = 0; i < Controllers.controllers.size(); ++i) {
            getController(i).poll();
        }
    }
    
    public static void clearEvents() {
        Controllers.events.clear();
    }
    
    public static boolean next() {
        if (Controllers.events.size() == 0) {
            Controllers.event = null;
            return false;
        }
        Controllers.event = Controllers.events.remove(0);
        return Controllers.event != null;
    }
    
    public static boolean isCreated() {
        return Controllers.created;
    }
    
    public static void destroy() {
    }
    
    public static org.lwjgl.input.Controller getEventSource() {
        return Controllers.event.getSource();
    }
    
    public static int getEventControlIndex() {
        return Controllers.event.getControlIndex();
    }
    
    public static boolean isEventButton() {
        return Controllers.event.isButton();
    }
    
    public static boolean isEventAxis() {
        return Controllers.event.isAxis();
    }
    
    public static boolean isEventXAxis() {
        return Controllers.event.isXAxis();
    }
    
    public static boolean isEventYAxis() {
        return Controllers.event.isYAxis();
    }
    
    public static boolean isEventPovX() {
        return Controllers.event.isPovX();
    }
    
    public static boolean isEventPovY() {
        return Controllers.event.isPovY();
    }
    
    public static long getEventNanoseconds() {
        return Controllers.event.getTimeStamp();
    }
    
    static void addEvent(final ControllerEvent event) {
        if (event != null) {
            Controllers.events.add(event);
        }
    }
    
    static {
        Controllers.controllers = new ArrayList();
        Controllers.events = new ArrayList();
    }
}
