package org.lwjgl.openal;

import java.nio.*;
import org.lwjgl.*;

public final class AL
{
    protected static ALCdevice device;
    protected static ALCcontext context;
    private static boolean created;
    static /* synthetic */ Class class$org$lwjgl$openal$AL;
    static /* synthetic */ Class class$org$lwjgl$openal$AL10;
    static /* synthetic */ Class class$org$lwjgl$openal$ALC10;
    static /* synthetic */ Class class$org$lwjgl$openal$ALC11;
    
    private static native void nCreate(final String p0) throws LWJGLException;
    
    private static native void nCreateDefault() throws LWJGLException;
    
    private static native void nDestroy();
    
    public static boolean isCreated() {
        return AL.created;
    }
    
    public static void create(final String deviceArguments, final int contextFrequency, final int contextRefresh, final boolean contextSynchronized) throws LWJGLException {
        create(deviceArguments, contextFrequency, contextRefresh, contextSynchronized, true);
    }
    
    public static void create(final String deviceArguments, final int contextFrequency, final int contextRefresh, final boolean contextSynchronized, final boolean openDevice) throws LWJGLException {
        if (AL.created) {
            throw new IllegalStateException("Only one OpenAL context may be instantiated at any one time.");
        }
        String libname = null;
        String[] library_names = null;
        switch (LWJGLUtil.getPlatform()) {
            case 3: {
                libname = "OpenAL32";
                library_names = new String[] { "OpenAL64.dll", "OpenAL32.dll" };
                break;
            }
            case 1: {
                libname = "openal";
                library_names = new String[] { "libopenal64.so", "libopenal.so", "libopenal.so.0" };
                break;
            }
            case 2: {
                libname = "openal";
                library_names = new String[] { "openal.dylib" };
                break;
            }
            default: {
                throw new LWJGLException("Unknown platform: " + LWJGLUtil.getPlatform());
            }
        }
        final String[] oalPaths = LWJGLUtil.getLibraryPaths(libname, library_names, ((AL.class$org$lwjgl$openal$AL == null) ? (AL.class$org$lwjgl$openal$AL = class$("org.lwjgl.openal.AL")) : AL.class$org$lwjgl$openal$AL).getClassLoader());
        LWJGLUtil.log("Found " + oalPaths.length + " OpenAL paths");
        int i = 0;
        while (i < oalPaths.length) {
            try {
                nCreate(oalPaths[i]);
                AL.created = true;
                init(deviceArguments, contextFrequency, contextRefresh, contextSynchronized, openDevice);
            }
            catch (LWJGLException e) {
                LWJGLUtil.log("Failed to load " + oalPaths[i] + ": " + e.getMessage());
                ++i;
                continue;
            }
            break;
        }
        if (!AL.created && LWJGLUtil.getPlatform() == 2) {
            nCreateDefault();
            AL.created = true;
            init(deviceArguments, contextFrequency, contextRefresh, contextSynchronized, openDevice);
        }
        if (!AL.created) {
            throw new LWJGLException("Could not locate OpenAL library.");
        }
    }
    
    private static void init(final String deviceArguments, final int contextFrequency, final int contextRefresh, final boolean contextSynchronized, final boolean openDevice) throws LWJGLException {
        try {
            AL10.initNativeStubs();
            ALC10.initNativeStubs();
            if (openDevice) {
                AL.device = ALC10.alcOpenDevice(deviceArguments);
                if (AL.device == null) {
                    throw new LWJGLException("Could not open ALC device");
                }
                if (contextFrequency == -1) {
                    AL.context = ALC10.alcCreateContext(AL.device, null);
                }
                else {
                    AL.context = ALC10.alcCreateContext(AL.device, ALCcontext.createAttributeList(contextFrequency, contextRefresh, (int)(contextSynchronized ? 1 : 0)));
                }
                ALC10.alcMakeContextCurrent(AL.context);
            }
        }
        catch (LWJGLException e) {
            destroy();
            throw e;
        }
        ALC11.initialize();
    }
    
    public static void create() throws LWJGLException {
        create(null, 44100, 60, false);
    }
    
    public static void destroy() {
        if (AL.context != null) {
            ALC10.alcMakeContextCurrent(null);
            ALC10.alcDestroyContext(AL.context);
            AL.context = null;
        }
        if (AL.device != null) {
            final boolean result = ALC10.alcCloseDevice(AL.device);
            AL.device = null;
        }
        resetNativeStubs((AL.class$org$lwjgl$openal$AL10 == null) ? (AL.class$org$lwjgl$openal$AL10 = class$("org.lwjgl.openal.AL10")) : AL.class$org$lwjgl$openal$AL10);
        resetNativeStubs((AL.class$org$lwjgl$openal$ALC10 == null) ? (AL.class$org$lwjgl$openal$ALC10 = class$("org.lwjgl.openal.ALC10")) : AL.class$org$lwjgl$openal$ALC10);
        resetNativeStubs((AL.class$org$lwjgl$openal$ALC11 == null) ? (AL.class$org$lwjgl$openal$ALC11 = class$("org.lwjgl.openal.ALC11")) : AL.class$org$lwjgl$openal$ALC11);
        if (AL.created) {
            nDestroy();
        }
        AL.created = false;
    }
    
    private static native void resetNativeStubs(final Class p0);
    
    public static ALCcontext getContext() {
        return AL.context;
    }
    
    public static ALCdevice getDevice() {
        return AL.device;
    }
    
    static /* synthetic */ Class class$(final String x0) {
        try {
            return Class.forName(x0);
        }
        catch (ClassNotFoundException x) {
            throw new NoClassDefFoundError().initCause(x);
        }
    }
    
    static {
        Sys.initialize();
    }
}
