package org.lwjgl.opengl;

import org.lwjgl.*;
import java.nio.charset.*;
import java.nio.*;

final class APIUtils
{
    private static final int INITIAL_BUFFER_SIZE = 256;
    private static final int INITIAL_LENGTHS_SIZE = 4;
    private static final int BUFFERS_SIZE = 32;
    private static final ThreadLocal arrayTL;
    private static final ThreadLocal bufferTL;
    private static final ThreadLocal lengthsTL;
    private static final ThreadLocal infiniteSeqTL;
    private static final ThreadLocal buffersTL;
    private static CharsetEncoder encoder;
    
    private static char[] getArray(final int size) {
        char[] array = APIUtils.arrayTL.get();
        if (array.length < size) {
            for (int sizeNew = array.length << 1; sizeNew < size; sizeNew <<= 1) {}
            array = new char[size];
            APIUtils.arrayTL.set(array);
        }
        return array;
    }
    
    static ByteBuffer getBufferByte(final int size) {
        ByteBuffer buffer = APIUtils.bufferTL.get();
        if (buffer.capacity() < size) {
            for (int sizeNew = buffer.capacity() << 1; sizeNew < size; sizeNew <<= 1) {}
            buffer = BufferUtils.createByteBuffer(size);
            APIUtils.bufferTL.set(buffer);
        }
        else {
            buffer.clear();
        }
        return buffer;
    }
    
    private static ByteBuffer getBufferByteOffset(final int size) {
        ByteBuffer buffer = APIUtils.bufferTL.get();
        if (buffer.capacity() < size) {
            for (int sizeNew = buffer.capacity() << 1; sizeNew < size; sizeNew <<= 1) {}
            final ByteBuffer bufferNew = BufferUtils.createByteBuffer(size);
            bufferNew.put(buffer);
            APIUtils.bufferTL.set(buffer = bufferNew);
        }
        else {
            buffer.position(buffer.limit());
            buffer.limit(buffer.capacity());
        }
        return buffer;
    }
    
    static ShortBuffer getBufferShort() {
        return APIUtils.buffersTL.get().shorts;
    }
    
    static IntBuffer getBufferInt() {
        return APIUtils.buffersTL.get().ints;
    }
    
    static LongBuffer getBufferLong() {
        return APIUtils.buffersTL.get().longs;
    }
    
    static FloatBuffer getBufferFloat() {
        return APIUtils.buffersTL.get().floats;
    }
    
    static DoubleBuffer getBufferDouble() {
        return APIUtils.buffersTL.get().doubles;
    }
    
    static IntBuffer getLengths() {
        return getLengths(1);
    }
    
    static IntBuffer getLengths(final int size) {
        IntBuffer lengths = APIUtils.lengthsTL.get();
        if (lengths.capacity() < size) {
            for (int sizeNew = lengths.capacity(); sizeNew < size; sizeNew <<= 1) {}
            lengths = BufferUtils.createIntBuffer(size);
            APIUtils.lengthsTL.set(lengths);
        }
        else {
            lengths.clear();
        }
        return lengths;
    }
    
    private static InfiniteCharSequence getInfiniteSeq() {
        return APIUtils.infiniteSeqTL.get();
    }
    
    private static void encode(final ByteBuffer buffer, final CharSequence string) {
        final InfiniteCharSequence infiniteSeq = getInfiniteSeq();
        infiniteSeq.setString(string);
        APIUtils.encoder.encode(infiniteSeq.buffer, buffer, true);
    }
    
    static String getString(final ByteBuffer buffer) {
        final int length = buffer.remaining();
        final char[] charArray = getArray(length);
        for (int i = buffer.position(); i < buffer.limit(); ++i) {
            charArray[i - buffer.position()] = (char)buffer.get(i);
        }
        return new String(charArray, 0, length);
    }
    
    static ByteBuffer getBuffer(final CharSequence string) {
        final ByteBuffer buffer = getBufferByte(string.length());
        encode(buffer, string);
        buffer.flip();
        return buffer;
    }
    
    static ByteBuffer getBuffer(final CharSequence string, final int offset) {
        final ByteBuffer buffer = getBufferByteOffset(offset + string.length());
        encode(buffer, string);
        buffer.flip();
        return buffer;
    }
    
    static ByteBuffer getBufferNT(final CharSequence string) {
        final ByteBuffer buffer = getBufferByte(string.length() + 1);
        encode(buffer, string);
        buffer.put((byte)0);
        buffer.flip();
        return buffer;
    }
    
    static int getTotalLength(final CharSequence[] strings) {
        int length = 0;
        for (int i = 0; i < strings.length; ++i) {
            length += strings[i].length();
        }
        return length;
    }
    
    static ByteBuffer getBuffer(final CharSequence[] strings) {
        final ByteBuffer buffer = getBufferByte(getTotalLength(strings));
        final InfiniteCharSequence infiniteSeq = getInfiniteSeq();
        for (int i = 0; i < strings.length; ++i) {
            infiniteSeq.setString(strings[i]);
            APIUtils.encoder.encode(infiniteSeq.buffer, buffer, true);
        }
        infiniteSeq.clear();
        buffer.flip();
        return buffer;
    }
    
    static ByteBuffer getBufferNT(final CharSequence[] strings) {
        final ByteBuffer buffer = getBufferByte(getTotalLength(strings) + strings.length);
        final InfiniteCharSequence infiniteSeq = getInfiniteSeq();
        for (int i = 0; i < strings.length; ++i) {
            infiniteSeq.setString(strings[i]);
            APIUtils.encoder.encode(infiniteSeq.buffer, buffer, true);
            buffer.put((byte)0);
        }
        infiniteSeq.clear();
        buffer.flip();
        return buffer;
    }
    
    static IntBuffer getLengths(final CharSequence[] strings) {
        final IntBuffer buffer = getLengths(strings.length);
        for (int i = 0; i < strings.length; ++i) {
            buffer.put(strings[i].length());
        }
        buffer.flip();
        return buffer;
    }
    
    static {
        arrayTL = new ThreadLocal() {
            protected Object initialValue() {
                return new char[256];
            }
        };
        bufferTL = new ThreadLocal() {
            protected Object initialValue() {
                return BufferUtils.createByteBuffer(256);
            }
        };
        lengthsTL = new ThreadLocal() {
            protected Object initialValue() {
                return BufferUtils.createIntBuffer(4);
            }
        };
        infiniteSeqTL = new ThreadLocal() {
            protected Object initialValue() {
                return new InfiniteCharSequence();
            }
        };
        buffersTL = new ThreadLocal() {
            protected Object initialValue() {
                return new Buffers();
            }
        };
        APIUtils.encoder = Charset.forName("US-ASCII").newEncoder();
    }
    
    private static class InfiniteCharSequence implements CharSequence
    {
        final CharBuffer buffer;
        CharSequence string;
        
        InfiniteCharSequence() {
            this.buffer = CharBuffer.wrap(this);
        }
        
        void setString(final CharSequence string) {
            this.string = string;
            this.buffer.position(0);
            this.buffer.limit(string.length());
        }
        
        void clear() {
            this.string = null;
        }
        
        public int length() {
            return Integer.MAX_VALUE;
        }
        
        public char charAt(final int index) {
            return this.string.charAt(index);
        }
        
        public CharSequence subSequence(final int start, final int end) {
            return this.string.subSequence(start, end);
        }
    }
    
    private static class Buffers
    {
        final ShortBuffer shorts;
        final IntBuffer ints;
        final LongBuffer longs;
        final FloatBuffer floats;
        final DoubleBuffer doubles;
        
        Buffers() {
            this.shorts = BufferUtils.createShortBuffer(32);
            this.ints = BufferUtils.createIntBuffer(32);
            this.longs = BufferUtils.createLongBuffer(32);
            this.floats = BufferUtils.createFloatBuffer(32);
            this.doubles = BufferUtils.createDoubleBuffer(32);
        }
    }
}
