package org.lwjgl.opengl;

import org.lwjgl.*;
import java.nio.*;

public class ARBBufferObject
{
    public static final int GL_STREAM_DRAW_ARB = 35040;
    public static final int GL_STREAM_READ_ARB = 35041;
    public static final int GL_STREAM_COPY_ARB = 35042;
    public static final int GL_STATIC_DRAW_ARB = 35044;
    public static final int GL_STATIC_READ_ARB = 35045;
    public static final int GL_STATIC_COPY_ARB = 35046;
    public static final int GL_DYNAMIC_DRAW_ARB = 35048;
    public static final int GL_DYNAMIC_READ_ARB = 35049;
    public static final int GL_DYNAMIC_COPY_ARB = 35050;
    public static final int GL_READ_ONLY_ARB = 35000;
    public static final int GL_WRITE_ONLY_ARB = 35001;
    public static final int GL_READ_WRITE_ARB = 35002;
    public static final int GL_BUFFER_SIZE_ARB = 34660;
    public static final int GL_BUFFER_USAGE_ARB = 34661;
    public static final int GL_BUFFER_ACCESS_ARB = 35003;
    public static final int GL_BUFFER_MAPPED_ARB = 35004;
    public static final int GL_BUFFER_MAP_POINTER_ARB = 35005;
    
    public static void glBindBufferARB(final int target, final int buffer) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glBindBufferARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        StateTracker.bindBuffer(caps, target, buffer);
        nglBindBufferARB(target, buffer, function_pointer);
    }
    
    private static native void nglBindBufferARB(final int p0, final int p1, final long p2);
    
    public static void glDeleteBuffersARB(final IntBuffer buffers) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glDeleteBuffersARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(buffers);
        nglDeleteBuffersARB(buffers.remaining(), buffers, buffers.position(), function_pointer);
    }
    
    private static native void nglDeleteBuffersARB(final int p0, final IntBuffer p1, final int p2, final long p3);
    
    public static void glDeleteBuffersARB(final int buffer) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glDeleteBuffersARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglDeleteBuffersARB(1, APIUtils.getBufferInt().put(0, buffer), 0, function_pointer);
    }
    
    public static void glGenBuffersARB(final IntBuffer buffers) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glGenBuffersARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(buffers);
        nglGenBuffersARB(buffers.remaining(), buffers, buffers.position(), function_pointer);
    }
    
    private static native void nglGenBuffersARB(final int p0, final IntBuffer p1, final int p2, final long p3);
    
    public static int glGenBuffersARB() {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glGenBuffersARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final IntBuffer buffers = APIUtils.getBufferInt();
        nglGenBuffersARB(1, buffers, buffers.position(), function_pointer);
        return buffers.get(0);
    }
    
    public static boolean glIsBufferARB(final int buffer) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glIsBufferARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final boolean __result = nglIsBufferARB(buffer, function_pointer);
        return __result;
    }
    
    private static native boolean nglIsBufferARB(final int p0, final long p1);
    
    public static void glBufferDataARB(final int target, final long size, final int usage) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glBufferDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglBufferDataARB(target, size, null, 0, usage, function_pointer);
    }
    
    public static void glBufferDataARB(final int target, final ByteBuffer data, final int usage) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glBufferDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglBufferDataARB(target, data.remaining(), data, data.position(), usage, function_pointer);
    }
    
    public static void glBufferDataARB(final int target, final DoubleBuffer data, final int usage) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glBufferDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglBufferDataARB(target, data.remaining() << 3, data, data.position() << 3, usage, function_pointer);
    }
    
    public static void glBufferDataARB(final int target, final FloatBuffer data, final int usage) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glBufferDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglBufferDataARB(target, data.remaining() << 2, data, data.position() << 2, usage, function_pointer);
    }
    
    public static void glBufferDataARB(final int target, final IntBuffer data, final int usage) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glBufferDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglBufferDataARB(target, data.remaining() << 2, data, data.position() << 2, usage, function_pointer);
    }
    
    public static void glBufferDataARB(final int target, final ShortBuffer data, final int usage) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glBufferDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglBufferDataARB(target, data.remaining() << 1, data, data.position() << 1, usage, function_pointer);
    }
    
    private static native void nglBufferDataARB(final int p0, final long p1, final Buffer p2, final int p3, final int p4, final long p5);
    
    public static void glBufferSubDataARB(final int target, final long offset, final ByteBuffer data) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glBufferSubDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglBufferSubDataARB(target, offset, data.remaining(), data, data.position(), function_pointer);
    }
    
    public static void glBufferSubDataARB(final int target, final long offset, final DoubleBuffer data) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glBufferSubDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglBufferSubDataARB(target, offset, data.remaining() << 3, data, data.position() << 3, function_pointer);
    }
    
    public static void glBufferSubDataARB(final int target, final long offset, final FloatBuffer data) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glBufferSubDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglBufferSubDataARB(target, offset, data.remaining() << 2, data, data.position() << 2, function_pointer);
    }
    
    public static void glBufferSubDataARB(final int target, final long offset, final IntBuffer data) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glBufferSubDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglBufferSubDataARB(target, offset, data.remaining() << 2, data, data.position() << 2, function_pointer);
    }
    
    public static void glBufferSubDataARB(final int target, final long offset, final ShortBuffer data) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glBufferSubDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglBufferSubDataARB(target, offset, data.remaining() << 1, data, data.position() << 1, function_pointer);
    }
    
    private static native void nglBufferSubDataARB(final int p0, final long p1, final long p2, final Buffer p3, final int p4, final long p5);
    
    public static void glGetBufferSubDataARB(final int target, final long offset, final ByteBuffer data) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glGetBufferSubDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglGetBufferSubDataARB(target, offset, data.remaining(), data, data.position(), function_pointer);
    }
    
    public static void glGetBufferSubDataARB(final int target, final long offset, final DoubleBuffer data) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glGetBufferSubDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglGetBufferSubDataARB(target, offset, data.remaining() << 3, data, data.position() << 3, function_pointer);
    }
    
    public static void glGetBufferSubDataARB(final int target, final long offset, final FloatBuffer data) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glGetBufferSubDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglGetBufferSubDataARB(target, offset, data.remaining() << 2, data, data.position() << 2, function_pointer);
    }
    
    public static void glGetBufferSubDataARB(final int target, final long offset, final IntBuffer data) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glGetBufferSubDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglGetBufferSubDataARB(target, offset, data.remaining() << 2, data, data.position() << 2, function_pointer);
    }
    
    public static void glGetBufferSubDataARB(final int target, final long offset, final ShortBuffer data) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glGetBufferSubDataARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(data);
        nglGetBufferSubDataARB(target, offset, data.remaining() << 1, data, data.position() << 1, function_pointer);
    }
    
    private static native void nglGetBufferSubDataARB(final int p0, final long p1, final long p2, final Buffer p3, final int p4, final long p5);
    
    public static ByteBuffer glMapBufferARB(final int target, final int access, final ByteBuffer old_buffer) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glMapBufferARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        if (old_buffer != null) {
            BufferChecks.checkDirect(old_buffer);
        }
        final ByteBuffer __result = nglMapBufferARB(target, access, GLChecks.getBufferObjectSizeARB(caps, target), old_buffer, function_pointer);
        return __result;
    }
    
    public static ByteBuffer glMapBufferARB(final int target, final int access, final long length, final ByteBuffer old_buffer) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glMapBufferARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        if (old_buffer != null) {
            BufferChecks.checkDirect(old_buffer);
        }
        final ByteBuffer __result = nglMapBufferARB(target, access, length, old_buffer, function_pointer);
        return __result;
    }
    
    private static native ByteBuffer nglMapBufferARB(final int p0, final int p1, final long p2, final ByteBuffer p3, final long p4);
    
    public static boolean glUnmapBufferARB(final int target) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glUnmapBufferARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final boolean __result = nglUnmapBufferARB(target, function_pointer);
        return __result;
    }
    
    private static native boolean nglUnmapBufferARB(final int p0, final long p1);
    
    public static void glGetBufferParameterARB(final int target, final int pname, final IntBuffer params) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glGetBufferParameterivARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkBuffer(params, 4);
        nglGetBufferParameterivARB(target, pname, params, params.position(), function_pointer);
    }
    
    private static native void nglGetBufferParameterivARB(final int p0, final int p1, final IntBuffer p2, final int p3, final long p4);
    
    public static int glGetBufferParameterARB(final int target, final int pname) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glGetBufferParameterivARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final IntBuffer params = APIUtils.getBufferInt();
        nglGetBufferParameterivARB(target, pname, params, params.position(), function_pointer);
        return params.get(0);
    }
    
    public static ByteBuffer glGetBufferPointerARB(final int target, final int pname) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_buffer_object_glGetBufferPointervARB_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final ByteBuffer __result = nglGetBufferPointervARB(target, pname, GLChecks.getBufferObjectSizeARB(caps, target), function_pointer);
        return __result;
    }
    
    private static native ByteBuffer nglGetBufferPointervARB(final int p0, final int p1, final long p2, final long p3);
}
