package org.lwjgl.opengl;

import org.lwjgl.*;
import java.nio.*;

public final class ARBSamplerObjects
{
    public static final int GL_SAMPLER_BINDING = 35097;
    
    public static void glGenSamplers(final IntBuffer samplers) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glGenSamplers_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(samplers);
        nglGenSamplers(samplers.remaining(), samplers, samplers.position(), function_pointer);
    }
    
    private static native void nglGenSamplers(final int p0, final IntBuffer p1, final int p2, final long p3);
    
    public static int glGenSamplers() {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glGenSamplers_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final IntBuffer samplers = APIUtils.getBufferInt();
        nglGenSamplers(1, samplers, samplers.position(), function_pointer);
        return samplers.get(0);
    }
    
    public static void glDeleteSamplers(final IntBuffer samplers) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glDeleteSamplers_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(samplers);
        nglDeleteSamplers(samplers.remaining(), samplers, samplers.position(), function_pointer);
    }
    
    private static native void nglDeleteSamplers(final int p0, final IntBuffer p1, final int p2, final long p3);
    
    public static void glDeleteSamplers(final int sampler) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glDeleteSamplers_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglDeleteSamplers(1, APIUtils.getBufferInt().put(0, sampler), 0, function_pointer);
    }
    
    public static boolean glIsSampler(final int sampler) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glIsSampler_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final boolean __result = nglIsSampler(sampler, function_pointer);
        return __result;
    }
    
    private static native boolean nglIsSampler(final int p0, final long p1);
    
    public static void glBindSampler(final int unit, final int sampler) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glBindSampler_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglBindSampler(unit, sampler, function_pointer);
    }
    
    private static native void nglBindSampler(final int p0, final int p1, final long p2);
    
    public static void glSamplerParameteri(final int sampler, final int pname, final int param) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glSamplerParameteri_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglSamplerParameteri(sampler, pname, param, function_pointer);
    }
    
    private static native void nglSamplerParameteri(final int p0, final int p1, final int p2, final long p3);
    
    public static void glSamplerParameterf(final int sampler, final int pname, final float param) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glSamplerParameterf_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglSamplerParameterf(sampler, pname, param, function_pointer);
    }
    
    private static native void nglSamplerParameterf(final int p0, final int p1, final float p2, final long p3);
    
    public static void glSamplerParameter(final int sampler, final int pname, final IntBuffer params) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glSamplerParameteriv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkBuffer(params, 4);
        nglSamplerParameteriv(sampler, pname, params, params.position(), function_pointer);
    }
    
    private static native void nglSamplerParameteriv(final int p0, final int p1, final IntBuffer p2, final int p3, final long p4);
    
    public static void glSamplerParameter(final int sampler, final int pname, final FloatBuffer params) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glSamplerParameterfv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkBuffer(params, 4);
        nglSamplerParameterfv(sampler, pname, params, params.position(), function_pointer);
    }
    
    private static native void nglSamplerParameterfv(final int p0, final int p1, final FloatBuffer p2, final int p3, final long p4);
    
    public static void glSamplerParameterI(final int sampler, final int pname, final IntBuffer params) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glSamplerParameterIiv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkBuffer(params, 4);
        nglSamplerParameterIiv(sampler, pname, params, params.position(), function_pointer);
    }
    
    private static native void nglSamplerParameterIiv(final int p0, final int p1, final IntBuffer p2, final int p3, final long p4);
    
    public static void glSamplerParameterIu(final int sampler, final int pname, final IntBuffer params) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glSamplerParameterIuiv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkBuffer(params, 4);
        nglSamplerParameterIuiv(sampler, pname, params, params.position(), function_pointer);
    }
    
    private static native void nglSamplerParameterIuiv(final int p0, final int p1, final IntBuffer p2, final int p3, final long p4);
    
    public static void glGetSamplerParameter(final int sampler, final int pname, final IntBuffer params) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glGetSamplerParameteriv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkBuffer(params, 4);
        nglGetSamplerParameteriv(sampler, pname, params, params.position(), function_pointer);
    }
    
    private static native void nglGetSamplerParameteriv(final int p0, final int p1, final IntBuffer p2, final int p3, final long p4);
    
    public static int glGetSamplerParameteri(final int sampler, final int pname) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glGetSamplerParameteriv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final IntBuffer params = APIUtils.getBufferInt();
        nglGetSamplerParameteriv(sampler, pname, params, params.position(), function_pointer);
        return params.get(0);
    }
    
    public static void glGetSamplerParameter(final int sampler, final int pname, final FloatBuffer params) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glGetSamplerParameterfv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkBuffer(params, 4);
        nglGetSamplerParameterfv(sampler, pname, params, params.position(), function_pointer);
    }
    
    private static native void nglGetSamplerParameterfv(final int p0, final int p1, final FloatBuffer p2, final int p3, final long p4);
    
    public static float glGetSamplerParameterf(final int sampler, final int pname) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glGetSamplerParameterfv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final FloatBuffer params = APIUtils.getBufferFloat();
        nglGetSamplerParameterfv(sampler, pname, params, params.position(), function_pointer);
        return params.get(0);
    }
    
    public static void glGetSamplerParameterI(final int sampler, final int pname, final IntBuffer params) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glGetSamplerParameterIiv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkBuffer(params, 4);
        nglGetSamplerParameterIiv(sampler, pname, params, params.position(), function_pointer);
    }
    
    private static native void nglGetSamplerParameterIiv(final int p0, final int p1, final IntBuffer p2, final int p3, final long p4);
    
    public static int glGetSamplerParameterIi(final int sampler, final int pname) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glGetSamplerParameterIiv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final IntBuffer params = APIUtils.getBufferInt();
        nglGetSamplerParameterIiv(sampler, pname, params, params.position(), function_pointer);
        return params.get(0);
    }
    
    public static void glGetSamplerParameterIu(final int sampler, final int pname, final IntBuffer params) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glGetSamplerParameterIuiv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkBuffer(params, 4);
        nglGetSamplerParameterIuiv(sampler, pname, params, params.position(), function_pointer);
    }
    
    private static native void nglGetSamplerParameterIuiv(final int p0, final int p1, final IntBuffer p2, final int p3, final long p4);
    
    public static int glGetSamplerParameterIui(final int sampler, final int pname) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sampler_objects_glGetSamplerParameterIuiv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final IntBuffer params = APIUtils.getBufferInt();
        nglGetSamplerParameterIuiv(sampler, pname, params, params.position(), function_pointer);
        return params.get(0);
    }
}
