package org.lwjgl.opengl;

import org.lwjgl.*;
import java.nio.*;

public final class ARBSync
{
    public static final int GL_MAX_SERVER_WAIT_TIMEOUT = 37137;
    public static final int GL_OBJECT_TYPE = 37138;
    public static final int GL_SYNC_CONDITION = 37139;
    public static final int GL_SYNC_STATUS = 37140;
    public static final int GL_SYNC_FLAGS = 37141;
    public static final int GL_SYNC_FENCE = 37142;
    public static final int GL_SYNC_GPU_COMMANDS_COMPLETE = 37143;
    public static final int GL_UNSIGNALED = 37144;
    public static final int GL_SIGNALED = 37145;
    public static final int GL_SYNC_FLUSH_COMMANDS_BIT = 1;
    public static final long GL_TIMEOUT_IGNORED = -1L;
    public static final int GL_ALREADY_SIGNALED = 37146;
    public static final int GL_TIMEOUT_EXPIRED = 37147;
    public static final int GL_CONDITION_SATISFIED = 37148;
    public static final int GL_WAIT_FAILED = 37149;
    
    public static GLSync glFenceSync(final int condition, final int flags) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sync_glFenceSync_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final GLSync __result = new GLSync(nglFenceSync(condition, flags, function_pointer));
        return __result;
    }
    
    private static native long nglFenceSync(final int p0, final int p1, final long p2);
    
    public static boolean glIsSync(final GLSync sync) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sync_glIsSync_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final boolean __result = nglIsSync(sync.getPointer(), function_pointer);
        return __result;
    }
    
    private static native boolean nglIsSync(final long p0, final long p1);
    
    public static void glDeleteSync(final GLSync sync) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sync_glDeleteSync_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglDeleteSync(sync.getPointer(), function_pointer);
    }
    
    private static native void nglDeleteSync(final long p0, final long p1);
    
    public static int glClientWaitSync(final GLSync sync, final int flags, final long timeout) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sync_glClientWaitSync_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final int __result = nglClientWaitSync(sync.getPointer(), flags, timeout, function_pointer);
        return __result;
    }
    
    private static native int nglClientWaitSync(final long p0, final int p1, final long p2, final long p3);
    
    public static void glWaitSync(final GLSync sync, final int flags, final long timeout) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sync_glWaitSync_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglWaitSync(sync.getPointer(), flags, timeout, function_pointer);
    }
    
    private static native void nglWaitSync(final long p0, final int p1, final long p2, final long p3);
    
    public static void glGetInteger(final int pname, final LongBuffer params) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sync_glGetInteger64v_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkBuffer(params, 1);
        nglGetInteger64v(pname, params, params.position(), function_pointer);
    }
    
    private static native void nglGetInteger64v(final int p0, final LongBuffer p1, final int p2, final long p3);
    
    public static long glGetInteger(final int pname) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sync_glGetInteger64v_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final LongBuffer params = APIUtils.getBufferLong();
        nglGetInteger64v(pname, params, params.position(), function_pointer);
        return params.get(0);
    }
    
    public static void glGetSync(final GLSync sync, final int pname, final IntBuffer length, final IntBuffer values) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sync_glGetSynciv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        if (length != null) {
            BufferChecks.checkBuffer(length, 1);
        }
        BufferChecks.checkDirect(values);
        nglGetSynciv(sync.getPointer(), pname, values.remaining(), length, (length != null) ? length.position() : 0, values, values.position(), function_pointer);
    }
    
    private static native void nglGetSynciv(final long p0, final int p1, final int p2, final IntBuffer p3, final int p4, final IntBuffer p5, final int p6, final long p7);
    
    public static int glGetSync(final GLSync sync, final int pname) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_sync_glGetSynciv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final IntBuffer values = APIUtils.getBufferInt();
        nglGetSynciv(sync.getPointer(), pname, 1, null, 0, values, values.position(), function_pointer);
        return values.get(0);
    }
}
