package org.lwjgl.opengl;

import java.nio.*;
import org.lwjgl.*;

public final class ARBUniformBufferObject
{
    public static final int GL_UNIFORM_BUFFER = 35345;
    public static final int GL_UNIFORM_BUFFER_BINDING = 35368;
    public static final int GL_UNIFORM_BUFFER_START = 35369;
    public static final int GL_UNIFORM_BUFFER_SIZE = 35370;
    public static final int GL_MAX_VERTEX_UNIFORM_BLOCKS = 35371;
    public static final int GL_MAX_GEOMETRY_UNIFORM_BLOCKS = 35372;
    public static final int GL_MAX_FRAGMENT_UNIFORM_BLOCKS = 35373;
    public static final int GL_MAX_COMBINED_UNIFORM_BLOCKS = 35374;
    public static final int GL_MAX_UNIFORM_BUFFER_BINDINGS = 35375;
    public static final int GL_MAX_UNIFORM_BLOCK_SIZE = 35376;
    public static final int GL_MAX_COMBINED_VERTEX_UNIFORM_COMPONENTS = 35377;
    public static final int GL_MAX_COMBINED_GEOMETRY_UNIFORM_COMPONENTS = 35378;
    public static final int GL_MAX_COMBINED_FRAGMENT_UNIFORM_COMPONENTS = 35379;
    public static final int GL_UNIFORM_BUFFER_OFFSET_ALIGNMENT = 35380;
    public static final int GL_ACTIVE_UNIFORM_BLOCK_MAX_NAME_LENGTH = 35381;
    public static final int GL_ACTIVE_UNIFORM_BLOCKS = 35382;
    public static final int GL_UNIFORM_TYPE = 35383;
    public static final int GL_UNIFORM_SIZE = 35384;
    public static final int GL_UNIFORM_NAME_LENGTH = 35385;
    public static final int GL_UNIFORM_BLOCK_INDEX = 35386;
    public static final int GL_UNIFORM_OFFSET = 35387;
    public static final int GL_UNIFORM_ARRAY_STRIDE = 35388;
    public static final int GL_UNIFORM_MATRIX_STRIDE = 35389;
    public static final int GL_UNIFORM_IS_ROW_MAJOR = 35390;
    public static final int GL_UNIFORM_BLOCK_BINDING = 35391;
    public static final int GL_UNIFORM_BLOCK_DATA_SIZE = 35392;
    public static final int GL_UNIFORM_BLOCK_NAME_LENGTH = 35393;
    public static final int GL_UNIFORM_BLOCK_ACTIVE_UNIFORMS = 35394;
    public static final int GL_UNIFORM_BLOCK_ACTIVE_UNIFORM_INDICES = 35395;
    public static final int GL_UNIFORM_BLOCK_REFERENCED_BY_VERTEX_SHADER = 35396;
    public static final int GL_UNIFORM_BLOCK_REFERENCED_BY_GEOMETRY_SHADER = 35397;
    public static final int GL_UNIFORM_BLOCK_REFERENCED_BY_FRAGMENT_SHADER = 35398;
    public static final int GL_INVALID_INDEX = -1;
    
    public static void glGetUniformIndices(final int program, final ByteBuffer uniformNames, final IntBuffer uniformIndices) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glGetUniformIndices_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(uniformNames);
        BufferChecks.checkNullTerminated(uniformNames, uniformIndices.remaining());
        BufferChecks.checkDirect(uniformIndices);
        nglGetUniformIndices(program, uniformIndices.remaining(), uniformNames, uniformNames.position(), uniformIndices, uniformIndices.position(), function_pointer);
    }
    
    private static native void nglGetUniformIndices(final int p0, final int p1, final ByteBuffer p2, final int p3, final IntBuffer p4, final int p5, final long p6);
    
    public static void glGetUniformIndices(final int program, final CharSequence[] uniformNames, final IntBuffer uniformIndices) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glGetUniformIndices_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkBuffer(uniformIndices, uniformNames.length);
        nglGetUniformIndices(program, uniformNames.length, APIUtils.getBufferNT(uniformNames), 0, uniformIndices, uniformIndices.position(), function_pointer);
    }
    
    public static void glGetActiveUniforms(final int program, final IntBuffer uniformIndices, final int pname, final IntBuffer params) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glGetActiveUniformsiv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(uniformIndices);
        BufferChecks.checkBuffer(params, uniformIndices.remaining());
        nglGetActiveUniformsiv(program, uniformIndices.remaining(), uniformIndices, uniformIndices.position(), pname, params, params.position(), function_pointer);
    }
    
    private static native void nglGetActiveUniformsiv(final int p0, final int p1, final IntBuffer p2, final int p3, final int p4, final IntBuffer p5, final int p6, final long p7);
    
    public static int glGetActiveUniforms(final int program, final int uniformIndex, final int pname) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glGetActiveUniformsiv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final IntBuffer params = APIUtils.getBufferInt();
        nglGetActiveUniformsiv(program, 1, params.put(1, uniformIndex), 1, pname, params, params.position(), function_pointer);
        return params.get(0);
    }
    
    public static void glGetActiveUniformName(final int program, final int uniformIndex, final IntBuffer length, final ByteBuffer uniformName) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glGetActiveUniformName_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        if (length != null) {
            BufferChecks.checkBuffer(length, 1);
        }
        BufferChecks.checkDirect(uniformName);
        nglGetActiveUniformName(program, uniformIndex, uniformName.remaining(), length, (length != null) ? length.position() : 0, uniformName, uniformName.position(), function_pointer);
    }
    
    private static native void nglGetActiveUniformName(final int p0, final int p1, final int p2, final IntBuffer p3, final int p4, final ByteBuffer p5, final int p6, final long p7);
    
    public static String glGetActiveUniformName(final int program, final int uniformIndex, final int bufSize) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glGetActiveUniformName_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final IntBuffer uniformName_length = APIUtils.getLengths();
        final ByteBuffer uniformName = APIUtils.getBufferByte(bufSize);
        nglGetActiveUniformName(program, uniformIndex, bufSize, uniformName_length, 0, uniformName, uniformName.position(), function_pointer);
        uniformName.limit(uniformName_length.get(0));
        return APIUtils.getString(uniformName);
    }
    
    public static int glGetUniformBlockIndex(final int program, final ByteBuffer uniformBlockName) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glGetUniformBlockIndex_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(uniformBlockName);
        BufferChecks.checkNullTerminated(uniformBlockName);
        final int __result = nglGetUniformBlockIndex(program, uniformBlockName, uniformBlockName.position(), function_pointer);
        return __result;
    }
    
    private static native int nglGetUniformBlockIndex(final int p0, final ByteBuffer p1, final int p2, final long p3);
    
    public static int glGetUniformBlockIndex(final int program, final CharSequence uniformBlockName) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glGetUniformBlockIndex_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final int __result = nglGetUniformBlockIndex(program, APIUtils.getBufferNT(uniformBlockName), 0, function_pointer);
        return __result;
    }
    
    public static void glGetActiveUniformBlock(final int program, final int uniformBlockIndex, final int pname, final IntBuffer params) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glGetActiveUniformBlockiv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkBuffer(params, 16);
        nglGetActiveUniformBlockiv(program, uniformBlockIndex, pname, params, params.position(), function_pointer);
    }
    
    private static native void nglGetActiveUniformBlockiv(final int p0, final int p1, final int p2, final IntBuffer p3, final int p4, final long p5);
    
    public static int glGetActiveUniformBlock(final int program, final int uniformBlockIndex, final int pname) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glGetActiveUniformBlockiv_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final IntBuffer params = APIUtils.getBufferInt();
        nglGetActiveUniformBlockiv(program, uniformBlockIndex, pname, params, params.position(), function_pointer);
        return params.get(0);
    }
    
    public static void glGetActiveUniformBlockName(final int program, final int uniformBlockIndex, final IntBuffer length, final ByteBuffer uniformBlockName) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glGetActiveUniformBlockName_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        if (length != null) {
            BufferChecks.checkBuffer(length, 1);
        }
        BufferChecks.checkDirect(uniformBlockName);
        nglGetActiveUniformBlockName(program, uniformBlockIndex, uniformBlockName.remaining(), length, (length != null) ? length.position() : 0, uniformBlockName, uniformBlockName.position(), function_pointer);
    }
    
    private static native void nglGetActiveUniformBlockName(final int p0, final int p1, final int p2, final IntBuffer p3, final int p4, final ByteBuffer p5, final int p6, final long p7);
    
    public static String glGetActiveUniformBlockName(final int program, final int uniformBlockIndex, final int bufSize) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glGetActiveUniformBlockName_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final IntBuffer uniformBlockName_length = APIUtils.getLengths();
        final ByteBuffer uniformBlockName = APIUtils.getBufferByte(bufSize);
        nglGetActiveUniformBlockName(program, uniformBlockIndex, bufSize, uniformBlockName_length, 0, uniformBlockName, uniformBlockName.position(), function_pointer);
        uniformBlockName.limit(uniformBlockName_length.get(0));
        return APIUtils.getString(uniformBlockName);
    }
    
    public static void glBindBufferRange(final int target, final int index, final int buffer, final long offset, final long size) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glBindBufferRange_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglBindBufferRange(target, index, buffer, offset, size, function_pointer);
    }
    
    private static native void nglBindBufferRange(final int p0, final int p1, final int p2, final long p3, final long p4, final long p5);
    
    public static void glBindBufferBase(final int target, final int index, final int buffer) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glBindBufferBase_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglBindBufferBase(target, index, buffer, function_pointer);
    }
    
    private static native void nglBindBufferBase(final int p0, final int p1, final int p2, final long p3);
    
    public static void glGetInteger(final int value, final int index, final IntBuffer data) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glGetIntegeri_v_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkBuffer(data, 4);
        nglGetIntegeri_v(value, index, data, data.position(), function_pointer);
    }
    
    private static native void nglGetIntegeri_v(final int p0, final int p1, final IntBuffer p2, final int p3, final long p4);
    
    public static int glGetInteger(final int value, final int index) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glGetIntegeri_v_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        final IntBuffer data = APIUtils.getBufferInt();
        nglGetIntegeri_v(value, index, data, data.position(), function_pointer);
        return data.get(0);
    }
    
    public static void glUniformBlockBinding(final int program, final int uniformBlockIndex, final int uniformBlockBinding) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.ARB_uniform_buffer_object_glUniformBlockBinding_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglUniformBlockBinding(program, uniformBlockIndex, uniformBlockBinding, function_pointer);
    }
    
    private static native void nglUniformBlockBinding(final int p0, final int p1, final int p2, final long p3);
}
