package org.lwjgl.opengl;

import java.nio.*;
import java.awt.*;
import org.lwjgl.*;
import java.security.*;

final class AWTSurfaceLock
{
    private static final int WAIT_DELAY_MILLIS = 100;
    private final ByteBuffer lock_buffer;
    private boolean firstLockSucceeded;
    
    AWTSurfaceLock() {
        this.firstLockSucceeded = false;
        this.lock_buffer = createHandle();
    }
    
    private static native ByteBuffer createHandle();
    
    public ByteBuffer lockAndGetHandle(final Canvas component) throws LWJGLException {
        while (!this.privilegedLockAndInitHandle(component)) {
            LWJGLUtil.log("Could not get drawing surface info, retrying...");
            try {
                Thread.sleep(100L);
            }
            catch (InterruptedException e) {
                LWJGLUtil.log("Interrupted while retrying: " + e);
            }
        }
        return this.lock_buffer;
    }
    
    private boolean privilegedLockAndInitHandle(final Canvas component) throws LWJGLException {
        if (this.firstLockSucceeded) {
            return lockAndInitHandle(this.lock_buffer, component);
        }
        try {
            final Object result = AccessController.doPrivileged((PrivilegedExceptionAction<Object>)new PrivilegedExceptionAction() {
                public Object run() throws LWJGLException {
                    return lockAndInitHandle(AWTSurfaceLock.this.lock_buffer, component);
                }
            });
            return this.firstLockSucceeded = (boolean)result;
        }
        catch (PrivilegedActionException e) {
            throw (LWJGLException)e.getException();
        }
    }
    
    private static native boolean lockAndInitHandle(final ByteBuffer p0, final Canvas p1) throws LWJGLException;
    
    void unlock() throws LWJGLException {
        nUnlock(this.lock_buffer);
    }
    
    private static native void nUnlock(final ByteBuffer p0) throws LWJGLException;
}
