package org.lwjgl.opengl;

import org.lwjgl.*;
import java.nio.*;

public final class EXTTransformFeedback
{
    public static final int GL_TRANSFORM_FEEDBACK_BUFFER_EXT = 35982;
    public static final int GL_TRANSFORM_FEEDBACK_BUFFER_START_EXT = 35972;
    public static final int GL_TRANSFORM_FEEDBACK_BUFFER_SIZE_EXT = 35973;
    public static final int GL_TRANSFORM_FEEDBACK_BUFFER_BINDING_EXT = 35983;
    public static final int GL_INTERLEAVED_ATTRIBS_EXT = 35980;
    public static final int GL_SEPARATE_ATTRIBS_EXT = 35981;
    public static final int GL_PRIMITIVES_GENERATED_EXT = 35975;
    public static final int GL_TRANSFORM_FEEDBACK_PRIMITIVES_WRITTEN_EXT = 35976;
    public static final int GL_RASTERIZER_DISCARD_EXT = 35977;
    public static final int GL_MAX_TRANSFORM_FEEDBACK_INTERLEAVED_COMPONENTS_EXT = 35978;
    public static final int GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_ATTRIBS_EXT = 35979;
    public static final int GL_MAX_TRANSFORM_FEEDBACK_SEPARATE_COMPONENTS_EXT = 35968;
    public static final int GL_TRANSFORM_FEEDBACK_VARYINGS_EXT = 35971;
    public static final int GL_TRANSFORM_FEEDBACK_BUFFER_MODE_EXT = 35967;
    public static final int GL_TRANSFORM_FEEDBACK_VARYING_MAX_LENGTH_EXT = 35958;
    
    public static void glBindBufferRangeEXT(final int target, final int index, final int buffer, final long offset, final long size) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.EXT_transform_feedback_glBindBufferRangeEXT_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglBindBufferRangeEXT(target, index, buffer, offset, size, function_pointer);
    }
    
    private static native void nglBindBufferRangeEXT(final int p0, final int p1, final int p2, final long p3, final long p4, final long p5);
    
    public static void glBindBufferOffsetEXT(final int target, final int index, final int buffer, final long offset) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.EXT_transform_feedback_glBindBufferOffsetEXT_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglBindBufferOffsetEXT(target, index, buffer, offset, function_pointer);
    }
    
    private static native void nglBindBufferOffsetEXT(final int p0, final int p1, final int p2, final long p3, final long p4);
    
    public static void glBindBufferBaseEXT(final int target, final int index, final int buffer) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.EXT_transform_feedback_glBindBufferBaseEXT_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglBindBufferBaseEXT(target, index, buffer, function_pointer);
    }
    
    private static native void nglBindBufferBaseEXT(final int p0, final int p1, final int p2, final long p3);
    
    public static void glBeginTransformFeedbackEXT(final int primitiveMode) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.EXT_transform_feedback_glBeginTransformFeedbackEXT_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglBeginTransformFeedbackEXT(primitiveMode, function_pointer);
    }
    
    private static native void nglBeginTransformFeedbackEXT(final int p0, final long p1);
    
    public static void glEndTransformFeedbackEXT() {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.EXT_transform_feedback_glEndTransformFeedbackEXT_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglEndTransformFeedbackEXT(function_pointer);
    }
    
    private static native void nglEndTransformFeedbackEXT(final long p0);
    
    public static void glTransformFeedbackVaryingsEXT(final int program, final int count, final ByteBuffer varyings, final int bufferMode) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.EXT_transform_feedback_glTransformFeedbackVaryingsEXT_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkDirect(varyings);
        BufferChecks.checkNullTerminated(varyings, count);
        nglTransformFeedbackVaryingsEXT(program, count, varyings, varyings.position(), bufferMode, function_pointer);
    }
    
    private static native void nglTransformFeedbackVaryingsEXT(final int p0, final int p1, final ByteBuffer p2, final int p3, final int p4, final long p5);
    
    public static void glTransformFeedbackVaryingsEXT(final int program, final CharSequence[] varyings, final int bufferMode) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.EXT_transform_feedback_glTransformFeedbackVaryingsEXT_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        nglTransformFeedbackVaryingsEXT(program, varyings.length, APIUtils.getBufferNT(varyings), 0, bufferMode, function_pointer);
    }
    
    public static void glGetTransformFeedbackVaryingEXT(final int program, final int index, final IntBuffer length, final IntBuffer size, final IntBuffer type, final ByteBuffer name) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.EXT_transform_feedback_glGetTransformFeedbackVaryingEXT_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        if (length != null) {
            BufferChecks.checkBuffer(length, 1);
        }
        BufferChecks.checkBuffer(size, 1);
        BufferChecks.checkBuffer(type, 1);
        BufferChecks.checkDirect(name);
        nglGetTransformFeedbackVaryingEXT(program, index, name.remaining(), length, (length != null) ? length.position() : 0, size, size.position(), type, type.position(), name, name.position(), function_pointer);
    }
    
    private static native void nglGetTransformFeedbackVaryingEXT(final int p0, final int p1, final int p2, final IntBuffer p3, final int p4, final IntBuffer p5, final int p6, final IntBuffer p7, final int p8, final ByteBuffer p9, final int p10, final long p11);
    
    public static String glGetTransformFeedbackVaryingEXT(final int program, final int index, final int bufSize, final IntBuffer size, final IntBuffer type) {
        final ContextCapabilities caps = GLContext.getCapabilities();
        final long function_pointer = caps.EXT_transform_feedback_glGetTransformFeedbackVaryingEXT_pointer;
        BufferChecks.checkFunctionAddress(function_pointer);
        BufferChecks.checkBuffer(size, 1);
        BufferChecks.checkBuffer(type, 1);
        final IntBuffer name_length = APIUtils.getLengths();
        final ByteBuffer name = APIUtils.getBufferByte(bufSize);
        nglGetTransformFeedbackVaryingEXT(program, index, bufSize, name_length, 0, size, size.position(), type, type.position(), name, name.position(), function_pointer);
        name.limit(name_length.get(0));
        return APIUtils.getString(name);
    }
}
