package org.lwjgl.opengl;

import java.lang.reflect.*;
import java.security.*;
import java.awt.*;
import org.lwjgl.*;

final class LinuxCanvasImplementation implements AWTCanvasImplementation
{
    static int getScreenFromDevice(final GraphicsDevice device) throws LWJGLException {
        try {
            final Method getScreen_method = AccessController.doPrivileged((PrivilegedExceptionAction<Method>)new PrivilegedExceptionAction() {
                public Object run() throws Exception {
                    return device.getClass().getMethod("getScreen", (Class<?>[])null);
                }
            });
            final Integer screen = (Integer)getScreen_method.invoke(device, (Object[])null);
            return screen;
        }
        catch (Exception e) {
            throw new LWJGLException(e);
        }
    }
    
    private static int getVisualIDFromConfiguration(final GraphicsConfiguration configuration) throws LWJGLException {
        try {
            final Method getVisual_method = AccessController.doPrivileged((PrivilegedExceptionAction<Method>)new PrivilegedExceptionAction() {
                public Object run() throws Exception {
                    return configuration.getClass().getMethod("getVisual", (Class<?>[])null);
                }
            });
            final Integer visual = (Integer)getVisual_method.invoke(configuration, (Object[])null);
            return visual;
        }
        catch (Exception e) {
            throw new LWJGLException(e);
        }
    }
    
    public PeerInfo createPeerInfo(final Canvas component, final PixelFormat pixel_format) throws LWJGLException {
        return new LinuxAWTGLCanvasPeerInfo(component);
    }
    
    public GraphicsConfiguration findConfiguration(final GraphicsDevice device, final PixelFormat pixel_format) throws LWJGLException {
        try {
            final int screen = getScreenFromDevice(device);
            final int visual_id_matching_format = findVisualIDFromFormat(screen, pixel_format);
            final GraphicsConfiguration[] configurations = device.getConfigurations();
            for (int i = 0; i < configurations.length; ++i) {
                final int visual_id = getVisualIDFromConfiguration(configurations[i]);
                if (visual_id == visual_id_matching_format) {
                    return configurations[i];
                }
            }
        }
        catch (LWJGLException e) {
            LWJGLUtil.log("Got exception while trying to determine configuration: " + e);
        }
        return null;
    }
    
    private static int findVisualIDFromFormat(final int screen, final PixelFormat pixel_format) throws LWJGLException {
        try {
            LinuxDisplay.lockAWT();
            try {
                GLContext.loadOpenGLLibrary();
                try {
                    LinuxDisplay.incDisplay();
                    return nFindVisualIDFromFormat(LinuxDisplay.getDisplay(), screen, pixel_format);
                }
                finally {
                    LinuxDisplay.decDisplay();
                }
            }
            finally {
                GLContext.unloadOpenGLLibrary();
            }
        }
        finally {
            LinuxDisplay.unlockAWT();
        }
    }
    
    private static native int nFindVisualIDFromFormat(final long p0, final int p1, final PixelFormat p2) throws LWJGLException;
}
