package org.lwjgl.opengl;

import java.awt.*;
import java.awt.event.*;

final class MacOSXCanvasListener implements ComponentListener, HierarchyListener
{
    private final Canvas canvas;
    private int width;
    private int height;
    private boolean context_update;
    
    MacOSXCanvasListener(final Canvas canvas) {
        (this.canvas = canvas).addComponentListener(this);
        canvas.addHierarchyListener(this);
        this.setUpdate();
    }
    
    public void disableListeners() {
        EventQueue.invokeLater(new Runnable() {
            public void run() {
                MacOSXCanvasListener.this.canvas.removeComponentListener(MacOSXCanvasListener.this);
                MacOSXCanvasListener.this.canvas.removeHierarchyListener(MacOSXCanvasListener.this);
            }
        });
    }
    
    public boolean syncShouldUpdateContext() {
        final boolean should_update;
        synchronized (this) {
            should_update = this.context_update;
            this.context_update = false;
        }
        return should_update;
    }
    
    private synchronized void setUpdate() {
        synchronized (this) {
            this.width = this.canvas.getWidth();
            this.height = this.canvas.getHeight();
            this.context_update = true;
        }
    }
    
    public int syncGetWidth() {
        synchronized (this) {
            return this.width;
        }
    }
    
    public int syncGetHeight() {
        synchronized (this) {
            return this.height;
        }
    }
    
    public void componentShown(final ComponentEvent e) {
    }
    
    public void componentHidden(final ComponentEvent e) {
    }
    
    public void componentResized(final ComponentEvent e) {
        this.setUpdate();
    }
    
    public void componentMoved(final ComponentEvent e) {
        this.setUpdate();
    }
    
    public void hierarchyChanged(final HierarchyEvent e) {
        this.setUpdate();
    }
}
