package org.lwjgl.opengl;

import java.nio.*;
import java.awt.*;
import org.lwjgl.*;

final class MacOSXMouseEventQueue extends MouseEventQueue
{
    private final IntBuffer delta_buffer;
    private boolean skip_event;
    private static boolean is_grabbed;
    
    MacOSXMouseEventQueue(final Component component) {
        super(component);
        this.delta_buffer = BufferUtils.createIntBuffer(2);
    }
    
    public void setGrabbed(final boolean grab) {
        super.setGrabbed(grab);
        this.warpCursor();
        grabMouse(grab);
    }
    
    private static synchronized void grabMouse(final boolean grab) {
        if (MacOSXMouseEventQueue.is_grabbed != grab && !(MacOSXMouseEventQueue.is_grabbed = grab)) {
            nGrabMouse(grab);
        }
    }
    
    protected void resetCursorToCenter() {
        super.resetCursorToCenter();
        getMouseDeltas(this.delta_buffer);
    }
    
    protected void updateDeltas(final long nanos) {
        super.updateDeltas(nanos);
        synchronized (this) {
            getMouseDeltas(this.delta_buffer);
            final int dx = this.delta_buffer.get(0);
            final int dy = -this.delta_buffer.get(1);
            if (this.skip_event) {
                this.skip_event = false;
                nGrabMouse(this.isGrabbed());
                return;
            }
            if (dx != 0 || dy != 0) {
                this.putMouseEventWithCoords((byte)(-1), (byte)0, dx, dy, 0, nanos);
                this.addDelta(dx, dy);
            }
        }
    }
    
    void warpCursor() {
        synchronized (this) {
            this.skip_event = this.isGrabbed();
        }
    }
    
    private static native void getMouseDeltas(final IntBuffer p0);
    
    private static native void nWarpCursor(final int p0, final int p1);
    
    static native void nGrabMouse(final boolean p0);
}
