package org.lwjgl.opengl;

import java.awt.*;
import java.nio.*;
import org.lwjgl.input.*;
import org.lwjgl.*;

final class WindowsDisplay implements DisplayImplementation
{
    private static final int GAMMA_LENGTH = 256;
    private static final int WM_CANCELMODE = 31;
    private static final int WM_MOUSEMOVE = 512;
    private static final int WM_LBUTTONDOWN = 513;
    private static final int WM_LBUTTONUP = 514;
    private static final int WM_LBUTTONDBLCLK = 515;
    private static final int WM_RBUTTONDOWN = 516;
    private static final int WM_RBUTTONUP = 517;
    private static final int WM_RBUTTONDBLCLK = 518;
    private static final int WM_MBUTTONDOWN = 519;
    private static final int WM_MBUTTONUP = 520;
    private static final int WM_MBUTTONDBLCLK = 521;
    private static final int WM_MOUSEWHEEL = 522;
    private static final int WM_CAPTURECHANGED = 533;
    private static final int WM_MOUSELEAVE = 675;
    private static final int WM_KEYDOWN = 256;
    private static final int WM_KEYUP = 257;
    private static final int WM_SYSKEYUP = 261;
    private static final int WM_SYSKEYDOWN = 260;
    private static final int WM_SYSCHAR = 262;
    private static final int WM_CHAR = 258;
    private static final int WM_SETICON = 128;
    private static final int WM_QUIT = 18;
    private static final int WM_SYSCOMMAND = 274;
    private static final int WM_PAINT = 15;
    private static final int WM_KILLFOCUS = 8;
    private static final int WM_SETFOCUS = 7;
    private static final int SC_SIZE = 61440;
    private static final int SC_MOVE = 61456;
    private static final int SC_MINIMIZE = 61472;
    private static final int SC_MAXIMIZE = 61488;
    private static final int SC_NEXTWINDOW = 61504;
    private static final int SC_PREVWINDOW = 61520;
    private static final int SC_CLOSE = 61536;
    private static final int SC_VSCROLL = 61552;
    private static final int SC_HSCROLL = 61568;
    private static final int SC_MOUSEMENU = 61584;
    private static final int SC_KEYMENU = 61696;
    private static final int SC_ARRANGE = 61712;
    private static final int SC_RESTORE = 61728;
    private static final int SC_TASKLIST = 61744;
    private static final int SC_SCREENSAVE = 61760;
    private static final int SC_HOTKEY = 61776;
    private static final int SC_DEFAULT = 61792;
    private static final int SC_MONITORPOWER = 61808;
    private static final int SC_CONTEXTHELP = 61824;
    private static final int SC_SEPARATOR = 61455;
    static final int SM_CXCURSOR = 13;
    static final int SM_CYCURSOR = 14;
    static final int SM_CMOUSEBUTTONS = 43;
    static final int SM_MOUSEWHEELPRESENT = 75;
    private static final int SIZE_RESTORED = 0;
    private static final int SIZE_MINIMIZED = 1;
    private static final int SIZE_MAXIMIZED = 2;
    private static final int WM_SIZE = 5;
    private static final int WM_ACTIVATE = 6;
    private static final int WA_INACTIVE = 0;
    private static final int WA_ACTIVE = 1;
    private static final int WA_CLICKACTIVE = 2;
    private static final int SW_SHOWMINNOACTIVE = 7;
    private static final int SW_SHOWDEFAULT = 10;
    private static final int SW_RESTORE = 9;
    private static final int ICON_SMALL = 0;
    private static final int ICON_BIG = 1;
    private static final IntBuffer rect_buffer;
    private static final Rect rect;
    private static final Rect rect2;
    private static WindowsDisplay current_display;
    private static boolean cursor_clipped;
    private WindowsDisplayPeerInfo peer_info;
    private Object current_cursor;
    private Canvas parent;
    private static boolean hasParent;
    private WindowsKeyboard keyboard;
    private WindowsMouse mouse;
    private boolean close_requested;
    private boolean is_dirty;
    private ByteBuffer current_gamma;
    private ByteBuffer saved_gamma;
    private DisplayMode current_mode;
    private boolean mode_set;
    private boolean isMinimized;
    private boolean isFocused;
    private boolean did_maximize;
    private boolean inAppActivate;
    private long hwnd;
    private long hdc;
    private long small_icon;
    private long large_icon;
    private int captureMouse;
    private boolean trackingMouse;
    private boolean mouseInside;
    
    WindowsDisplay() {
        this.captureMouse = -1;
        this.trackingMouse = false;
        this.mouseInside = false;
        WindowsDisplay.current_display = this;
    }
    
    public void createWindow(final DisplayMode mode, final Canvas parent, final int x, final int y) throws LWJGLException {
        this.close_requested = false;
        this.is_dirty = false;
        this.isMinimized = false;
        this.isFocused = false;
        this.did_maximize = false;
        this.parent = parent;
        WindowsDisplay.hasParent = (parent != null);
        final long parent_hwnd = (parent != null) ? getHwnd(parent) : 0L;
        this.hwnd = nCreateWindow(x, y, mode.getWidth(), mode.getHeight(), Display.isFullscreen() || isUndecorated(), parent != null, parent_hwnd);
        if (this.hwnd == 0L) {
            throw new LWJGLException("Failed to create window");
        }
        this.hdc = getDC(this.hwnd);
        if (this.hdc == 0L) {
            nDestroyWindow(this.hwnd);
            throw new LWJGLException("Failed to get dc");
        }
        try {
            final int format = WindowsPeerInfo.choosePixelFormat(this.getHdc(), 0, 0, this.peer_info.getPixelFormat(), null, true, true, false, true);
            WindowsPeerInfo.setPixelFormat(this.getHdc(), format);
            this.peer_info.initDC(this.getHwnd(), this.getHdc());
            showWindow(this.getHwnd(), 10);
            if (parent == null) {
                setForegroundWindow(this.getHwnd());
                setFocus(this.getHwnd());
            }
        }
        catch (LWJGLException e) {
            nReleaseDC(this.hwnd, this.hdc);
            nDestroyWindow(this.hwnd);
            throw e;
        }
    }
    
    private static native long nCreateWindow(final int p0, final int p1, final int p2, final int p3, final boolean p4, final boolean p5, final long p6) throws LWJGLException;
    
    private static boolean isUndecorated() {
        return Display.getPrivilegedBoolean("org.lwjgl.opengl.Window.undecorated");
    }
    
    private static long getHwnd(final Canvas parent) throws LWJGLException {
        final AWTCanvasImplementation awt_impl = AWTGLCanvas.createImplementation();
        final WindowsPeerInfo parent_peer_info = (WindowsPeerInfo)awt_impl.createPeerInfo(parent, null);
        final ByteBuffer parent_peer_info_handle = parent_peer_info.lockAndGetHandle();
        try {
            return parent_peer_info.getHwnd();
        }
        finally {
            parent_peer_info.unlock();
        }
    }
    
    public void destroyWindow() {
        nReleaseDC(this.hwnd, this.hdc);
        nDestroyWindow(this.hwnd);
        this.freeLargeIcon();
        this.freeSmallIcon();
        resetCursorClipping();
    }
    
    private static native void nReleaseDC(final long p0, final long p1);
    
    private static native void nDestroyWindow(final long p0);
    
    static void resetCursorClipping() {
        if (WindowsDisplay.cursor_clipped) {
            try {
                clipCursor(null);
            }
            catch (LWJGLException e) {
                LWJGLUtil.log("Failed to reset cursor clipping: " + e);
            }
            WindowsDisplay.cursor_clipped = false;
        }
    }
    
    private static void getGlobalClientRect(final long hwnd, final Rect rect) {
        WindowsDisplay.rect_buffer.put(0, 0).put(1, 0);
        clientToScreen(hwnd, WindowsDisplay.rect_buffer);
        final int offset_x = WindowsDisplay.rect_buffer.get(0);
        final int offset_y = WindowsDisplay.rect_buffer.get(1);
        getClientRect(hwnd, WindowsDisplay.rect_buffer);
        rect.copyFromBuffer(WindowsDisplay.rect_buffer);
        rect.offset(offset_x, offset_y);
    }
    
    static void setupCursorClipping(final long hwnd) throws LWJGLException {
        WindowsDisplay.cursor_clipped = true;
        getGlobalClientRect(hwnd, WindowsDisplay.rect);
        WindowsDisplay.rect.copyToBuffer(WindowsDisplay.rect_buffer);
        clipCursor(WindowsDisplay.rect_buffer);
    }
    
    private static native void clipCursor(final IntBuffer p0) throws LWJGLException;
    
    public void switchDisplayMode(final DisplayMode mode) throws LWJGLException {
        nSwitchDisplayMode(mode);
        this.current_mode = mode;
        this.mode_set = true;
    }
    
    private static native void nSwitchDisplayMode(final DisplayMode p0) throws LWJGLException;
    
    private void appActivate(final boolean active) {
        if (this.inAppActivate) {
            return;
        }
        this.inAppActivate = true;
        this.isFocused = active;
        if (active) {
            if (Display.isFullscreen()) {
                this.restoreDisplayMode();
            }
            if (this.parent == null) {
                showWindow(this.getHwnd(), 9);
                setForegroundWindow(this.getHwnd());
                setFocus(this.getHwnd());
            }
            this.did_maximize = true;
            if (Display.isFullscreen()) {
                this.updateClipping();
            }
        }
        else if (Display.isFullscreen()) {
            showWindow(this.getHwnd(), 7);
            this.resetDisplayMode();
        }
        else {
            this.updateClipping();
        }
        this.updateCursor();
        this.inAppActivate = false;
    }
    
    private static native void showWindow(final long p0, final int p1);
    
    private static native void setForegroundWindow(final long p0);
    
    private static native void setFocus(final long p0);
    
    private void restoreDisplayMode() {
        try {
            this.doSetGammaRamp(this.current_gamma);
        }
        catch (LWJGLException e) {
            LWJGLUtil.log("Failed to restore gamma: " + e.getMessage());
        }
        if (!this.mode_set) {
            this.mode_set = true;
            try {
                nSwitchDisplayMode(this.current_mode);
            }
            catch (LWJGLException e) {
                LWJGLUtil.log("Failed to restore display mode: " + e.getMessage());
            }
        }
    }
    
    public void resetDisplayMode() {
        try {
            this.doSetGammaRamp(this.saved_gamma);
        }
        catch (LWJGLException e) {
            LWJGLUtil.log("Failed to reset gamma ramp: " + e.getMessage());
        }
        this.current_gamma = this.saved_gamma;
        if (this.mode_set) {
            this.mode_set = false;
            nResetDisplayMode();
        }
        resetCursorClipping();
    }
    
    private static native void nResetDisplayMode();
    
    public int getGammaRampLength() {
        return 256;
    }
    
    public void setGammaRamp(final FloatBuffer gammaRamp) throws LWJGLException {
        this.doSetGammaRamp(convertToNativeRamp(gammaRamp));
    }
    
    private static native ByteBuffer convertToNativeRamp(final FloatBuffer p0) throws LWJGLException;
    
    private static native ByteBuffer getCurrentGammaRamp() throws LWJGLException;
    
    private void doSetGammaRamp(final ByteBuffer native_gamma) throws LWJGLException {
        nSetGammaRamp(native_gamma);
        this.current_gamma = native_gamma;
    }
    
    private static native void nSetGammaRamp(final ByteBuffer p0) throws LWJGLException;
    
    public String getAdapter() {
        try {
            final String maxObjNo = WindowsRegistry.queryRegistrationKey(3, "HARDWARE\\DeviceMap\\Video", "MaxObjectNumber");
            final int maxObjectNumber = maxObjNo.charAt(0);
            String vga_driver_value = "";
            for (int i = 0; i < maxObjectNumber; ++i) {
                final String adapter_string = WindowsRegistry.queryRegistrationKey(3, "HARDWARE\\DeviceMap\\Video", "\\Device\\Video" + i);
                final String root_key = "\\registry\\machine\\";
                if (adapter_string.toLowerCase().startsWith(root_key)) {
                    final String driver_value = WindowsRegistry.queryRegistrationKey(3, adapter_string.substring(root_key.length()), "InstalledDisplayDrivers");
                    if (driver_value.toUpperCase().startsWith("VGA")) {
                        vga_driver_value = driver_value;
                    }
                    else if (!driver_value.toUpperCase().startsWith("RDP") && !driver_value.toUpperCase().startsWith("NMNDD")) {
                        return driver_value;
                    }
                }
            }
            if (!vga_driver_value.equals("")) {
                return vga_driver_value;
            }
        }
        catch (LWJGLException e) {
            LWJGLUtil.log("Exception occurred while querying registry: " + e);
        }
        return null;
    }
    
    public String getVersion() {
        final String driver = this.getAdapter();
        if (driver != null) {
            final String[] drivers = driver.split(",");
            if (drivers.length > 0) {
                final WindowsFileVersion version = this.nGetVersion(drivers[0] + ".dll");
                if (version != null) {
                    return version.toString();
                }
            }
        }
        return null;
    }
    
    private native WindowsFileVersion nGetVersion(final String p0);
    
    public DisplayMode init() throws LWJGLException {
        final ByteBuffer currentGammaRamp = getCurrentGammaRamp();
        this.saved_gamma = currentGammaRamp;
        this.current_gamma = currentGammaRamp;
        return this.current_mode = getCurrentDisplayMode();
    }
    
    private static native DisplayMode getCurrentDisplayMode() throws LWJGLException;
    
    public void setTitle(final String title) {
        nSetTitle(this.hwnd, title);
    }
    
    private static native void nSetTitle(final long p0, final String p1);
    
    public boolean isCloseRequested() {
        final boolean saved = this.close_requested;
        this.close_requested = false;
        return saved;
    }
    
    public boolean isVisible() {
        return !this.isMinimized;
    }
    
    public boolean isActive() {
        return this.isFocused;
    }
    
    public boolean isDirty() {
        final boolean saved = this.is_dirty;
        this.is_dirty = false;
        return saved;
    }
    
    public PeerInfo createPeerInfo(final PixelFormat pixel_format) throws LWJGLException {
        return this.peer_info = new WindowsDisplayPeerInfo(pixel_format);
    }
    
    public void update() {
        nUpdate();
        if (this.parent != null && this.parent.isFocusOwner()) {
            setFocus(this.getHwnd());
        }
        if (this.did_maximize) {
            this.did_maximize = false;
            try {
                if (Display.getDrawable().getContext() != null && Display.getDrawable().getContext().isCurrent()) {
                    Display.getDrawable().getContext().makeCurrent();
                }
            }
            catch (LWJGLException e) {
                LWJGLUtil.log("Exception occurred while trying to make context current: " + e);
            }
        }
    }
    
    private static native void nUpdate();
    
    public void reshape(final int x, final int y, final int width, final int height) {
        nReshape(this.getHwnd(), x, y, width, height, Display.isFullscreen() || isUndecorated(), this.parent != null);
    }
    
    private static native void nReshape(final long p0, final int p1, final int p2, final int p3, final int p4, final boolean p5, final boolean p6);
    
    public native DisplayMode[] getAvailableDisplayModes() throws LWJGLException;
    
    public boolean hasWheel() {
        return this.mouse.hasWheel();
    }
    
    public int getButtonCount() {
        return this.mouse.getButtonCount();
    }
    
    public void createMouse() throws LWJGLException {
        this.mouse = new WindowsMouse(this.getHwnd());
    }
    
    public void destroyMouse() {
        if (this.mouse != null) {
            this.mouse.destroy();
        }
        this.mouse = null;
    }
    
    public void pollMouse(final IntBuffer coord_buffer, final ByteBuffer buttons) {
        this.mouse.poll(coord_buffer, buttons);
    }
    
    public void readMouse(final ByteBuffer buffer) {
        this.mouse.read(buffer);
    }
    
    public void grabMouse(final boolean grab) {
        this.mouse.grab(grab, this.shouldGrab());
        this.updateCursor();
    }
    
    public int getNativeCursorCapabilities() {
        return 1;
    }
    
    public void setCursorPosition(final int x, final int y) {
        getGlobalClientRect(this.getHwnd(), WindowsDisplay.rect);
        final int transformed_x = WindowsDisplay.rect.left + x;
        final int transformed_y = WindowsDisplay.rect.bottom - 1 - y;
        nSetCursorPosition(transformed_x, transformed_y);
        this.setMousePosition(x, y);
    }
    
    private static native void nSetCursorPosition(final int p0, final int p1);
    
    public void setNativeCursor(final Object handle) throws LWJGLException {
        this.current_cursor = handle;
        this.updateCursor();
    }
    
    private void updateCursor() {
        try {
            if (this.mouse != null && this.shouldGrab()) {
                nSetNativeCursor(this.getHwnd(), this.mouse.getBlankCursor());
            }
            else {
                nSetNativeCursor(this.getHwnd(), this.current_cursor);
            }
        }
        catch (LWJGLException e) {
            LWJGLUtil.log("Failed to update cursor: " + e);
        }
    }
    
    static native void nSetNativeCursor(final long p0, final Object p1) throws LWJGLException;
    
    public int getMinCursorSize() {
        return getSystemMetrics(13);
    }
    
    public int getMaxCursorSize() {
        return getSystemMetrics(13);
    }
    
    static native int getSystemMetrics(final int p0);
    
    private static native long getDllInstance();
    
    private long getHwnd() {
        return this.hwnd;
    }
    
    private long getHdc() {
        return this.hdc;
    }
    
    private static native long getDC(final long p0);
    
    private static native long getDesktopWindow();
    
    private static native long getForegroundWindow();
    
    static void centerCursor(final long hwnd) {
        if (getForegroundWindow() != hwnd && !WindowsDisplay.hasParent) {
            return;
        }
        getGlobalClientRect(hwnd, WindowsDisplay.rect);
        final int local_offset_x = WindowsDisplay.rect.left;
        final int local_offset_y = WindowsDisplay.rect.top;
        final int center_x = (WindowsDisplay.rect.left + WindowsDisplay.rect.right) / 2;
        final int center_y = (WindowsDisplay.rect.top + WindowsDisplay.rect.bottom) / 2;
        nSetCursorPosition(center_x, center_y);
        final int local_x = center_x - local_offset_x;
        final int local_y = center_y - local_offset_y;
        if (WindowsDisplay.current_display != null) {
            WindowsDisplay.current_display.setMousePosition(local_x, transformY(hwnd, local_y));
        }
    }
    
    private void setMousePosition(final int x, final int y) {
        if (this.mouse != null) {
            this.mouse.setPosition(x, y);
        }
    }
    
    public void createKeyboard() throws LWJGLException {
        this.keyboard = new WindowsKeyboard(this.getHwnd());
    }
    
    public void destroyKeyboard() {
        this.keyboard.destroy();
        this.keyboard = null;
    }
    
    public void pollKeyboard(final ByteBuffer keyDownBuffer) {
        this.keyboard.poll(keyDownBuffer);
    }
    
    public void readKeyboard(final ByteBuffer buffer) {
        this.keyboard.read(buffer);
    }
    
    public static native ByteBuffer nCreateCursor(final int p0, final int p1, final int p2, final int p3, final int p4, final IntBuffer p5, final int p6, final IntBuffer p7, final int p8) throws LWJGLException;
    
    public Object createCursor(final int width, final int height, final int xHotspot, final int yHotspot, final int numImages, final IntBuffer images, final IntBuffer delays) throws LWJGLException {
        return doCreateCursor(width, height, xHotspot, yHotspot, numImages, images, delays);
    }
    
    static Object doCreateCursor(final int width, final int height, final int xHotspot, final int yHotspot, final int numImages, final IntBuffer images, final IntBuffer delays) throws LWJGLException {
        return nCreateCursor(width, height, xHotspot, yHotspot, numImages, images, images.position(), delays, (delays != null) ? delays.position() : -1);
    }
    
    public void destroyCursor(final Object cursorHandle) {
        doDestroyCursor(cursorHandle);
    }
    
    static native void doDestroyCursor(final Object p0);
    
    public int getPbufferCapabilities() {
        try {
            return this.nGetPbufferCapabilities(new PixelFormat(0, 0, 0, 0, 0, 0, 0, 0, false));
        }
        catch (LWJGLException e) {
            LWJGLUtil.log("Exception occurred while determining pbuffer capabilities: " + e);
            return 0;
        }
    }
    
    private native int nGetPbufferCapabilities(final PixelFormat p0) throws LWJGLException;
    
    public boolean isBufferLost(final PeerInfo handle) {
        return ((WindowsPbufferPeerInfo)handle).isBufferLost();
    }
    
    public PeerInfo createPbuffer(final int width, final int height, final PixelFormat pixel_format, final IntBuffer pixelFormatCaps, final IntBuffer pBufferAttribs) throws LWJGLException {
        return new WindowsPbufferPeerInfo(width, height, pixel_format, pixelFormatCaps, pBufferAttribs);
    }
    
    public void setPbufferAttrib(final PeerInfo handle, final int attrib, final int value) {
        ((WindowsPbufferPeerInfo)handle).setPbufferAttrib(attrib, value);
    }
    
    public void bindTexImageToPbuffer(final PeerInfo handle, final int buffer) {
        ((WindowsPbufferPeerInfo)handle).bindTexImageToPbuffer(buffer);
    }
    
    public void releaseTexImageFromPbuffer(final PeerInfo handle, final int buffer) {
        ((WindowsPbufferPeerInfo)handle).releaseTexImageFromPbuffer(buffer);
    }
    
    private void freeSmallIcon() {
        if (this.small_icon != 0L) {
            destroyIcon(this.small_icon);
            this.small_icon = 0L;
        }
    }
    
    private void freeLargeIcon() {
        if (this.large_icon != 0L) {
            destroyIcon(this.large_icon);
            this.large_icon = 0L;
        }
    }
    
    public int setIcon(final ByteBuffer[] icons) {
        boolean done_small = false;
        boolean done_large = false;
        int used = 0;
        final int small_icon_size = 16;
        final int large_icon_size = 32;
        for (int i = 0; i < icons.length; ++i) {
            final int size = icons[i].limit() / 4;
            if ((int)Math.sqrt(size) == small_icon_size && !done_small) {
                this.freeSmallIcon();
                this.small_icon = createIcon(small_icon_size, small_icon_size, icons[i].asIntBuffer());
                sendMessage(this.hwnd, 128L, 0L, this.small_icon);
                ++used;
                done_small = true;
            }
            if ((int)Math.sqrt(size) == large_icon_size && !done_large) {
                this.freeLargeIcon();
                this.large_icon = createIcon(large_icon_size, large_icon_size, icons[i].asIntBuffer());
                sendMessage(this.hwnd, 128L, 1L, this.large_icon);
                ++used;
                done_large = true;
            }
        }
        return used;
    }
    
    private static native long createIcon(final int p0, final int p1, final IntBuffer p2);
    
    private static native void destroyIcon(final long p0);
    
    private static native long sendMessage(final long p0, final long p1, final long p2, final long p3);
    
    private void handleMouseButton(final int button, final int state, final long millis) {
        if (this.mouse != null) {
            this.mouse.handleMouseButton((byte)button, (byte)state, millis);
            if (this.captureMouse == -1 && button != -1 && state == 1) {
                this.captureMouse = button;
                nSetCapture(this.hwnd);
            }
            if (this.captureMouse != -1 && button == this.captureMouse && state == 0) {
                this.captureMouse = -1;
                nReleaseCapture();
            }
        }
        if (this.parent != null && !this.isFocused) {
            setFocus(this.getHwnd());
        }
    }
    
    private boolean shouldGrab() {
        return !this.isMinimized && this.isFocused && Mouse.isGrabbed();
    }
    
    private void handleMouseMoved(final int x, final int y, final long millis) {
        if (this.mouse != null) {
            this.mouse.handleMouseMoved(x, y, millis, this.shouldGrab());
        }
    }
    
    private static native long nSetCapture(final long p0);
    
    private static native boolean nReleaseCapture();
    
    private void handleMouseScrolled(final int amount, final long millis) {
        if (this.mouse != null) {
            this.mouse.handleMouseScrolled(amount, millis);
        }
    }
    
    private static native void getClientRect(final long p0, final IntBuffer p1);
    
    private void handleChar(final long wParam, final long lParam, final long millis) {
        final byte previous_state = (byte)(lParam >>> 30 & 0x1L);
        final byte state = (byte)(1L - (lParam >>> 31 & 0x1L));
        final boolean repeat = state == previous_state;
        if (this.keyboard != null) {
            this.keyboard.handleChar((int)(wParam & 0xFFL), millis, repeat);
        }
    }
    
    private void handleKeyButton(final long wParam, final long lParam, final long millis) {
        final byte previous_state = (byte)(lParam >>> 30 & 0x1L);
        final byte state = (byte)(1L - (lParam >>> 31 & 0x1L));
        final boolean repeat = state == previous_state;
        final byte extended = (byte)(lParam >>> 24 & 0x1L);
        final int scan_code = (int)(lParam >>> 16 & 0xFFL);
        if (this.keyboard != null) {
            this.keyboard.handleKey((int)wParam, scan_code, extended != 0, state, millis, repeat);
            if (this.captureMouse != -1 && this.keyboard.isKeyDown(1)) {
                nReleaseCapture();
                this.captureMouse = -1;
            }
        }
    }
    
    private static int transformY(final long hwnd, final int y) {
        getClientRect(hwnd, WindowsDisplay.rect_buffer);
        WindowsDisplay.rect.copyFromBuffer(WindowsDisplay.rect_buffer);
        return WindowsDisplay.rect.bottom - WindowsDisplay.rect.top - 1 - y;
    }
    
    private static native void clientToScreen(final long p0, final IntBuffer p1);
    
    private static int handleMessage(final long hwnd, final int msg, final long wParam, final long lParam, final long millis) {
        if (WindowsDisplay.current_display != null) {
            return WindowsDisplay.current_display.doHandleMessage(hwnd, msg, wParam, lParam, millis);
        }
        return defWindowProc(hwnd, msg, wParam, lParam);
    }
    
    private static native int defWindowProc(final long p0, final int p1, final long p2, final long p3);
    
    private void checkCursorState() {
        this.updateClipping();
    }
    
    private void updateClipping() {
        Label_0093: {
            if ((Display.isFullscreen() || (this.mouse != null && this.mouse.isGrabbed())) && !this.isMinimized && this.isFocused) {
                if (getForegroundWindow() != this.getHwnd()) {
                    if (!WindowsDisplay.hasParent) {
                        break Label_0093;
                    }
                }
                try {
                    setupCursorClipping(this.getHwnd());
                }
                catch (LWJGLException e) {
                    LWJGLUtil.log("setupCursorClipping failed: " + e.getMessage());
                }
                return;
            }
        }
        resetCursorClipping();
    }
    
    private void setMinimized(final boolean m) {
        this.isMinimized = m;
        this.checkCursorState();
    }
    
    private int doHandleMessage(final long hwnd, final int msg, final long wParam, final long lParam, final long millis) {
        switch (msg) {
            case 6: {
                switch ((int)wParam) {
                    case 1:
                    case 2: {
                        this.appActivate(true);
                        break;
                    }
                    case 0: {
                        this.appActivate(false);
                        break;
                    }
                }
                return 0;
            }
            case 5: {
                switch ((int)wParam) {
                    case 0:
                    case 2: {
                        this.setMinimized(false);
                        break;
                    }
                    case 1: {
                        this.setMinimized(true);
                        break;
                    }
                }
                return defWindowProc(hwnd, msg, wParam, lParam);
            }
            case 8: {
                this.appActivate(false);
                return 0;
            }
            case 7: {
                this.appActivate(true);
                return 0;
            }
            case 512: {
                final int xPos = (short)(lParam & 0xFFFFL);
                final int yPos = transformY(this.getHwnd(), (short)(lParam >> 16 & 0xFFFFL));
                this.handleMouseMoved(xPos, yPos, millis);
                this.checkCursorState();
                this.mouseInside = true;
                if (!this.trackingMouse) {
                    this.trackingMouse = this.nTrackMouseEvent(hwnd);
                }
                return 0;
            }
            case 522: {
                final int dwheel = (short)(wParam >> 16 & 0xFFFFL);
                this.handleMouseScrolled(dwheel, millis);
                return 0;
            }
            case 513: {
                this.handleMouseButton(0, 1, millis);
                return 0;
            }
            case 514: {
                this.handleMouseButton(0, 0, millis);
                return 0;
            }
            case 516: {
                this.handleMouseButton(1, 1, millis);
                return 0;
            }
            case 517: {
                this.handleMouseButton(1, 0, millis);
                return 0;
            }
            case 519: {
                this.handleMouseButton(2, 1, millis);
                return 0;
            }
            case 520: {
                this.handleMouseButton(2, 0, millis);
                return 0;
            }
            case 258:
            case 262: {
                this.handleChar(wParam, lParam, millis);
                return 0;
            }
            case 257:
            case 261: {
                if (wParam == 44L && this.keyboard != null && !this.keyboard.isKeyDown(183)) {
                    long fake_lparam = lParam & 0x7FFFFFFFL;
                    fake_lparam &= 0xFFFFFFFFBFFFFFFFL;
                    this.handleKeyButton(wParam, fake_lparam, millis);
                }
            }
            case 256:
            case 260: {
                this.handleKeyButton(wParam, lParam, millis);
                return defWindowProc(hwnd, msg, wParam, lParam);
            }
            case 18: {
                this.close_requested = true;
                return 0;
            }
            case 274: {
                switch ((int)(wParam & 0xFFF0L)) {
                    case 61584:
                    case 61696:
                    case 61760:
                    case 61808: {
                        return 0;
                    }
                    case 61536: {
                        this.close_requested = true;
                        return 0;
                    }
                    default: {
                        return defWindowProc(hwnd, msg, wParam, lParam);
                    }
                }
                break;
            }
            case 15: {
                this.is_dirty = true;
                return defWindowProc(hwnd, msg, wParam, lParam);
            }
            case 675: {
                this.mouseInside = false;
                this.trackingMouse = false;
                return defWindowProc(hwnd, msg, wParam, lParam);
            }
            case 31: {
                nReleaseCapture();
            }
            case 533: {
                if (this.captureMouse != -1) {
                    this.handleMouseButton(this.captureMouse, 0, millis);
                    this.captureMouse = -1;
                }
                return 0;
            }
            default: {
                return defWindowProc(hwnd, msg, wParam, lParam);
            }
        }
    }
    
    public int getWidth() {
        return Display.getDisplayMode().getWidth();
    }
    
    public int getHeight() {
        return Display.getDisplayMode().getHeight();
    }
    
    private int firstMouseButtonDown() {
        for (int i = 0; i < Mouse.getButtonCount(); ++i) {
            if (Mouse.isButtonDown(i)) {
                return i;
            }
        }
        return -1;
    }
    
    private native boolean nTrackMouseEvent(final long p0);
    
    public boolean isInsideWindow() {
        return this.mouseInside;
    }
    
    static {
        rect_buffer = BufferUtils.createIntBuffer(4);
        rect = new Rect();
        rect2 = new Rect();
        WindowsDisplay.hasParent = false;
    }
    
    private static final class Rect
    {
        public int top;
        public int bottom;
        public int left;
        public int right;
        
        public void copyToBuffer(final IntBuffer buffer) {
            buffer.put(0, this.top).put(1, this.bottom).put(2, this.left).put(3, this.right);
        }
        
        public void copyFromBuffer(final IntBuffer buffer) {
            this.top = buffer.get(0);
            this.bottom = buffer.get(1);
            this.left = buffer.get(2);
            this.right = buffer.get(3);
        }
        
        public void offset(final int offset_x, final int offset_y) {
            this.left += offset_x;
            this.right += offset_x;
            this.top += offset_y;
            this.bottom += offset_y;
        }
        
        public static void intersect(final Rect r1, final Rect r2, final Rect dst) {
            dst.top = Math.max(r1.top, r2.top);
            dst.bottom = Math.min(r1.bottom, r2.bottom);
            dst.left = Math.max(r1.left, r2.left);
            dst.right = Math.min(r1.right, r2.right);
        }
        
        public String toString() {
            return "Rect: top = " + this.top + " bottom = " + this.bottom + " left = " + this.left + " right = " + this.right;
        }
    }
}
