package org.lwjgl.opengl;

import java.util.regex.*;
import java.io.*;
import java.util.*;
import java.security.*;

public class XRandR
{
    private static Screen[] current;
    private static Map screens;
    
    private static void populate() {
        if (XRandR.screens == null) {
            XRandR.screens = new HashMap();
            try {
                final Process p = Runtime.getRuntime().exec(new String[] { "xrandr", "-q" });
                final List currentList = new ArrayList();
                final List possibles = new ArrayList();
                String name = null;
                final BufferedReader br = new BufferedReader(new InputStreamReader(p.getInputStream()));
                String line;
                while ((line = br.readLine()) != null) {
                    line = line.trim();
                    final String[] sa = line.split("\\s+");
                    if (sa[1].equals("connected")) {
                        if (name != null) {
                            XRandR.screens.put(name, possibles.toArray(new Screen[possibles.size()]));
                            possibles.clear();
                        }
                        name = sa[0];
                        currentList.add(new Screen(name, sa[2]));
                    }
                    else {
                        if (!Pattern.matches("\\d*x\\d*", sa[0])) {
                            continue;
                        }
                        possibles.add(new Screen(name, sa[0]));
                    }
                }
                XRandR.screens.put(name, possibles.toArray(new Screen[possibles.size()]));
                XRandR.current = currentList.toArray(new Screen[currentList.size()]);
            }
            catch (IOException e) {
                e.printStackTrace();
            }
        }
    }
    
    public static Screen[] getConfiguration() {
        AccessController.doPrivileged((PrivilegedAction<Object>)new PrivilegedAction() {
            public Object run() {
                populate();
                return null;
            }
        });
        return XRandR.current.clone();
    }
    
    public static void setConfiguration(final Screen[] screens) {
        if (screens.length == 0) {
            throw new IllegalArgumentException("Must specify at least one screen");
        }
        final List cmd = new ArrayList();
        cmd.add("xrandr");
        for (int i = 0; i < XRandR.current.length; ++i) {
            boolean found = false;
            for (int j = 0; j < screens.length; ++j) {
                if (screens[j].name.equals(XRandR.current[i].name)) {
                    found = true;
                    break;
                }
            }
            if (!found) {
                cmd.add("--output");
                cmd.add(XRandR.current[i].name);
                cmd.add("--off");
            }
        }
        for (int i = 0; i < screens.length; ++i) {
            screens[i].getArgs(cmd);
        }
        try {
            final Process p = Runtime.getRuntime().exec(cmd.toArray(new String[cmd.size()]));
            final BufferedReader br = new BufferedReader(new InputStreamReader(p.getInputStream()));
            String line;
            while ((line = br.readLine()) != null) {
                System.out.println(line);
            }
            XRandR.current = screens;
        }
        catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    public static String[] getScreenNames() {
        populate();
        return (String[])XRandR.screens.keySet().toArray(new String[XRandR.screens.size()]);
    }
    
    public static Screen[] getResolutions(final String name) {
        populate();
        return XRandR.screens.get(name).clone();
    }
    
    public static class Screen implements Cloneable
    {
        public final String name;
        public final int width;
        public final int height;
        public int xPos;
        public int yPos;
        
        private Screen(final String name, final String conf) {
            this.xPos = 0;
            this.yPos = 0;
            this.name = name;
            final String[] sa = conf.split("\\D");
            this.width = Integer.parseInt(sa[0]);
            this.height = Integer.parseInt(sa[1]);
            if (sa.length > 2) {
                this.xPos = Integer.parseInt(sa[2]);
                this.yPos = Integer.parseInt(sa[3]);
            }
        }
        
        private void getArgs(final List argList) {
            argList.add("--output");
            argList.add(this.name);
            argList.add("--mode");
            argList.add(this.width + "x" + this.height);
            argList.add("--pos");
            argList.add(this.xPos + "x" + this.yPos);
        }
        
        public String toString() {
            return this.name + " " + this.width + "x" + this.height + " @ " + this.xPos + "x" + this.yPos;
        }
    }
}
