package org.lwjgl.util;

import org.lwjgl.opengl.*;
import org.lwjgl.*;
import java.util.*;

public final class Display
{
    private static final boolean DEBUG = false;
    static /* synthetic */ Class class$org$lwjgl$util$Display;
    
    public static DisplayMode[] getAvailableDisplayModes(final int minWidth, final int minHeight, final int maxWidth, final int maxHeight, final int minBPP, final int maxBPP, final int minFreq, final int maxFreq) throws LWJGLException {
        final DisplayMode[] modes = org.lwjgl.opengl.Display.getAvailableDisplayModes();
        if (LWJGLUtil.DEBUG) {
            System.out.println("Available screen modes:");
            for (int i = 0; i < modes.length; ++i) {
                System.out.println(modes[i]);
            }
        }
        final ArrayList matches = new ArrayList(modes.length);
        for (int j = 0; j < modes.length; ++j) {
            assert modes[j] != null : "" + j + " " + modes.length;
            if (minWidth == -1 || modes[j].getWidth() >= minWidth) {
                if (maxWidth == -1 || modes[j].getWidth() <= maxWidth) {
                    if (minHeight == -1 || modes[j].getHeight() >= minHeight) {
                        if (maxHeight == -1 || modes[j].getHeight() <= maxHeight) {
                            if (minBPP == -1 || modes[j].getBitsPerPixel() >= minBPP) {
                                if (maxBPP == -1 || modes[j].getBitsPerPixel() <= maxBPP) {
                                    if (modes[j].getFrequency() != 0) {
                                        if (minFreq != -1 && modes[j].getFrequency() < minFreq) {
                                            continue;
                                        }
                                        if (maxFreq != -1 && modes[j].getFrequency() > maxFreq) {
                                            continue;
                                        }
                                    }
                                    matches.add(modes[j]);
                                }
                            }
                        }
                    }
                }
            }
        }
        final DisplayMode[] ret = new DisplayMode[matches.size()];
        matches.toArray(ret);
        if (LWJGLUtil.DEBUG) {}
        return ret;
    }
    
    public static DisplayMode setDisplayMode(final DisplayMode[] dm, final String[] param) throws Exception {
        Arrays.sort(dm, new Sorter(param));
        if (LWJGLUtil.DEBUG) {
            System.out.println("Sorted display modes:");
            for (int i = 0; i < dm.length; ++i) {
                System.out.println(dm[i]);
            }
        }
        int i = 0;
        while (i < dm.length) {
            try {
                if (LWJGLUtil.DEBUG) {
                    System.out.println("Attempting to set displaymode: " + dm[i]);
                }
                org.lwjgl.opengl.Display.setDisplayMode(dm[i]);
                return dm[i];
            }
            catch (Exception e) {
                if (LWJGLUtil.DEBUG) {
                    System.out.println("Failed to set display mode to " + dm[i]);
                    e.printStackTrace();
                }
                ++i;
                continue;
            }
            break;
        }
        throw new Exception("Failed to set display mode.");
    }
    
    static /* synthetic */ Class class$(final String x0) {
        try {
            return Class.forName(x0);
        }
        catch (ClassNotFoundException x) {
            throw new NoClassDefFoundError().initCause(x);
        }
    }
    
    static {
        $assertionsDisabled = !((Display.class$org$lwjgl$util$Display == null) ? (Display.class$org$lwjgl$util$Display = class$("org.lwjgl.util.Display")) : Display.class$org$lwjgl$util$Display).desiredAssertionStatus();
    }
    
    class Sorter implements Comparator
    {
        final FieldAccessor[] field;
        private final /* synthetic */ String[] val$param;
        
        Sorter(final String[] val$param) {
            this.val$param = val$param;
            this.field = new FieldAccessor[this.val$param.length];
            for (int i = 0; i < this.field.length; ++i) {
                final int idx = this.val$param[i].indexOf(61);
                if (idx > 0) {
                    this.field[i] = new FieldAccessor(this.val$param[i].substring(0, idx), 0, Integer.parseInt(this.val$param[i].substring(idx + 1, this.val$param[i].length())), true);
                }
                else if (this.val$param[i].charAt(0) == '-') {
                    this.field[i] = new FieldAccessor(this.val$param[i].substring(1), -1, 0, false);
                }
                else {
                    this.field[i] = new FieldAccessor(this.val$param[i], 1, 0, false);
                }
            }
        }
        
        public int compare(final Object o1, final Object o2) {
            final DisplayMode dm1 = (DisplayMode)o1;
            final DisplayMode dm2 = (DisplayMode)o2;
            for (int i = 0; i < this.field.length; ++i) {
                final int f1 = this.field[i].getInt(dm1);
                final int f2 = this.field[i].getInt(dm2);
                if (this.field[i].usePreferred && f1 != f2) {
                    if (f1 == this.field[i].preferred) {
                        return -1;
                    }
                    if (f2 == this.field[i].preferred) {
                        return 1;
                    }
                    final int absf1 = Math.abs(f1 - this.field[i].preferred);
                    final int absf2 = Math.abs(f2 - this.field[i].preferred);
                    if (absf1 < absf2) {
                        return -1;
                    }
                    if (absf1 > absf2) {
                        return 1;
                    }
                }
                else {
                    if (f1 < f2) {
                        return this.field[i].order;
                    }
                    if (f1 != f2) {
                        return -this.field[i].order;
                    }
                }
            }
            return 0;
        }
    }
}
