package org.lwjgl.util.jinput;

import java.util.*;
import java.lang.reflect.*;
import java.io.*;
import net.java.games.input.*;

final class LWJGLKeyboard extends Keyboard
{
    static /* synthetic */ Class class$org$lwjgl$input$Keyboard;
    
    protected LWJGLKeyboard() {
        super("LWJGLKeyboard", createComponents(), new Controller[0], new Rumbler[0]);
    }
    
    private static final Component[] createComponents() {
        final List components = new ArrayList();
        final Field[] vkey_fields = ((LWJGLKeyboard.class$org$lwjgl$input$Keyboard == null) ? (LWJGLKeyboard.class$org$lwjgl$input$Keyboard = class$("org.lwjgl.input.Keyboard")) : LWJGLKeyboard.class$org$lwjgl$input$Keyboard).getFields();
        for (int i = 0; i < vkey_fields.length; ++i) {
            final Field vkey_field = vkey_fields[i];
            try {
                if (Modifier.isStatic(vkey_field.getModifiers()) && vkey_field.getType() == Integer.TYPE && vkey_field.getName().startsWith("KEY_")) {
                    final int vkey_code = vkey_field.getInt(null);
                    final Component.Identifier.Key key_id = KeyMap.map(vkey_code);
                    if (key_id != Component.Identifier.Key.UNKNOWN) {
                        components.add(new Key(key_id, vkey_code));
                    }
                }
            }
            catch (IllegalAccessException e) {
                throw new RuntimeException(e);
            }
        }
        return components.toArray(new Component[0]);
    }
    
    public final synchronized void pollDevice() throws IOException {
        if (!org.lwjgl.input.Keyboard.isCreated()) {
            return;
        }
        org.lwjgl.input.Keyboard.poll();
        final Component[] components = this.getComponents();
        for (int i = 0; i < components.length; ++i) {
            final Key key = (Key)components[i];
            key.update();
        }
    }
    
    protected final synchronized boolean getNextDeviceEvent(final Event event) throws IOException {
        if (!org.lwjgl.input.Keyboard.isCreated()) {
            return false;
        }
        if (!org.lwjgl.input.Keyboard.next()) {
            return false;
        }
        final int lwjgl_key = org.lwjgl.input.Keyboard.getEventKey();
        if (lwjgl_key == 0) {
            return false;
        }
        final Component.Identifier.Key key_id = KeyMap.map(lwjgl_key);
        if (key_id == null) {
            return false;
        }
        final Component key = this.getComponent(key_id);
        if (key == null) {
            return false;
        }
        final float value = org.lwjgl.input.Keyboard.getEventKeyState() ? 1.0f : 0.0f;
        event.set(key, value, org.lwjgl.input.Keyboard.getEventNanoseconds());
        return true;
    }
    
    static /* synthetic */ Class class$(final String x0) {
        try {
            return Class.forName(x0);
        }
        catch (ClassNotFoundException x) {
            throw new NoClassDefFoundError().initCause(x);
        }
    }
    
    private static final class Key extends AbstractComponent
    {
        private final int lwjgl_key;
        private float value;
        
        public Key(final Component.Identifier.Key key_id, final int lwjgl_key) {
            super(key_id.getName(), key_id);
            this.lwjgl_key = lwjgl_key;
        }
        
        public final void update() {
            this.value = (org.lwjgl.input.Keyboard.isKeyDown(this.lwjgl_key) ? 1.0f : 0.0f);
        }
        
        protected final float poll() {
            return this.value;
        }
        
        public final boolean isRelative() {
            return false;
        }
        
        public final boolean isAnalog() {
            return false;
        }
    }
}
