package org.lwjgl.util.jinput;

import java.io.*;
import net.java.games.input.*;

final class LWJGLMouse extends Mouse
{
    private static final int EVENT_X = 1;
    private static final int EVENT_Y = 2;
    private static final int EVENT_WHEEL = 3;
    private static final int EVENT_BUTTON = 4;
    private static final int EVENT_DONE = 5;
    private int event_state;
    
    protected LWJGLMouse() {
        super("LWJGLMouse", createComponents(), new Controller[0], new Rumbler[0]);
        this.event_state = 5;
    }
    
    private static final Component[] createComponents() {
        return new Component[] { new Axis(Component.Identifier.Axis.X), new Axis(Component.Identifier.Axis.Y), new Axis(Component.Identifier.Axis.Z), new Button(Component.Identifier.Button.LEFT), new Button(Component.Identifier.Button.MIDDLE), new Button(Component.Identifier.Button.RIGHT) };
    }
    
    public final synchronized void pollDevice() throws IOException {
        if (!org.lwjgl.input.Mouse.isCreated()) {
            return;
        }
        org.lwjgl.input.Mouse.poll();
        for (int i = 0; i < 3; ++i) {
            this.setButtonState(i);
        }
    }
    
    private final Button map(final int lwjgl_button) {
        switch (lwjgl_button) {
            case 0: {
                return (Button)this.getLeft();
            }
            case 1: {
                return (Button)this.getRight();
            }
            case 2: {
                return (Button)this.getMiddle();
            }
            default: {
                return null;
            }
        }
    }
    
    private final void setButtonState(final int lwjgl_button) {
        final Button button = this.map(lwjgl_button);
        if (button != null) {
            button.setValue(org.lwjgl.input.Mouse.isButtonDown(lwjgl_button) ? 1.0f : 0.0f);
        }
    }
    
    protected final synchronized boolean getNextDeviceEvent(final Event event) throws IOException {
        if (!org.lwjgl.input.Mouse.isCreated()) {
            return false;
        }
        while (true) {
            final long nanos = org.lwjgl.input.Mouse.getEventNanoseconds();
            switch (this.event_state) {
                case 1: {
                    this.event_state = 2;
                    final int dx = org.lwjgl.input.Mouse.getEventDX();
                    if (dx != 0) {
                        event.set(this.getX(), dx, nanos);
                        return true;
                    }
                    continue;
                }
                case 2: {
                    this.event_state = 3;
                    final int dy = -org.lwjgl.input.Mouse.getEventDY();
                    if (dy != 0) {
                        event.set(this.getY(), dy, nanos);
                        return true;
                    }
                    continue;
                }
                case 3: {
                    this.event_state = 4;
                    final int dwheel = org.lwjgl.input.Mouse.getEventDWheel();
                    if (dwheel != 0) {
                        event.set(this.getWheel(), dwheel, nanos);
                        return true;
                    }
                    continue;
                }
                case 4: {
                    this.event_state = 5;
                    final int lwjgl_button = org.lwjgl.input.Mouse.getEventButton();
                    if (lwjgl_button == -1) {
                        continue;
                    }
                    final Button button = this.map(lwjgl_button);
                    if (button != null) {
                        event.set(button, org.lwjgl.input.Mouse.getEventButtonState() ? 1.0f : 0.0f, nanos);
                        return true;
                    }
                    continue;
                }
                case 5: {
                    if (!org.lwjgl.input.Mouse.next()) {
                        return false;
                    }
                    this.event_state = 1;
                    continue;
                }
            }
        }
    }
    
    static final class Axis extends AbstractComponent
    {
        public Axis(final Component.Identifier.Axis axis_id) {
            super(axis_id.getName(), axis_id);
        }
        
        public final boolean isRelative() {
            return true;
        }
        
        protected final float poll() throws IOException {
            return 0.0f;
        }
        
        public final boolean isAnalog() {
            return true;
        }
    }
    
    static final class Button extends AbstractComponent
    {
        private float value;
        
        public Button(final Component.Identifier.Button button_id) {
            super(button_id.getName(), button_id);
        }
        
        protected final void setValue(final float value) {
            this.value = value;
        }
        
        protected final float poll() throws IOException {
            return this.value;
        }
        
        public final boolean isRelative() {
            return false;
        }
        
        public final boolean isAnalog() {
            return false;
        }
    }
}
